#!/usr/bin/perl
#  Ana Sanz Merino [University of California Berkeley, 2003]

# Server Program
use IO::Socket::INET;
use POSIX qw(:sys_wait_h);

my $server_port = 20000;

# Create a new socket
my $server = IO::Socket::INET->new(LocalPort => $server_port,
                Proto     => 'udp',
                Reuse     => 1,)
    or die "Could not create udp server on port $server_port\n";

# Keep receiving messages from client
my $msg;
my $bufsize = 128;
my ($portaddr, $portno, $ipaddr);

my $PRINT_DEBUG = 0;

while (1) {
    $portaddr = recv($server, $buf, $bufsize, 0);
    ($portno, $ipaddr) = sockaddr_in($portaddr);
    if ($PRINT_DEBUG) {
        print "\nReceived message = '", $buf,"'\n";
        print "\n\nfrom $ipaddr:$portno\n";
    }
    @instr = split(' ', $buf);
    if ($instr[0] eq MAC_KEY_VERIFY) {
        if ($#instr < 2) {
            $msg = "Error: Invalid IP/KEY format";
        } else {
            $mac_addr = $instr[1];
            $session_key_md5 = $instr[2];
            if (exists $key{$mac_addr}) {
                if ($session_key_md5 eq $key{$mac_addr}) {
                    #### TODO ######################################
                    # Open firewall for the client if it is closed #
                    ################################################
                    $msg = "OK";
                } else  {
                    $msg = "NG (key doesn't match)";
                }
            } else {
                $msg = "NG (key not registered yet)";
            }
        }
    } elsif ($instr[0] eq MAC_KEY_ADD) {
        if ($#instr < 2) {
            $msg = "Error: Invalid MAC/KEY format";
        } else {
            $mac_addr = $instr[1];
            $session_key_md5 = $instr[2];
            $key{$mac_addr} = $session_key_md5;
            #### TODO ######################################
            # Close firewall for the client if it is open  #
            ################################################
            $msg = "OK (MAC/Key pair added)";
        }
    } elsif ($instr[0] eq MAC_KEY_DEL) {
        if ($#instr < 2) {
            $msg = "Error: Invalid MAC/KEY format";
        } else {
            $mac_addr = $instr[1];
            $session_key_md5 = $instr[2];
            if (exists $key{$mac_addr}) {
		# delete key anyway
		if ($session_key_md5 eq $key{$mac_addr}) {
		    $msg = "OK (MAC/Key pair deleted, key match)";
		} else {
		    $msg = "OK (MAC/Key pair deleted, key doesn't match)";
	        }
                delete($key{$mac_addr});
            } else {
                $msg = "NG (MAC/Key not found)";
            }
        }
    } else {
	$msg = "NG (Invalid Command Type)";
    }
    if ($PRINT_DEBUG) {
        print "Result = $msg\n";
    }
    send($server, $msg, 0, $portaddr);
}
