/*
 * $Id: Config.java,v 1.10.2.2 2002/09/17 01:52:20 jsergent Exp $
 * Config.java
 *
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.experimentalstuff.com/Technologies/IPL/download.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2002 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): Ana Sanz Merino [University of California Berkeley, 2003]
 */

package com.sun.liberty;
import java.util.Properties;

/**
 * Config parameters for the system.
 * Typically initialized by ConfigServlet from parameters listed in the
 * web-app deployment descriptor retrieved via ServletContext.
 *
 * See comments in the default deployment descriptor for further information
 * on these parameters.  Documentation is there to avoid duplication and
 * since the information there is visible at deployment time.
 *
 * @author Jonathan Sergent
 * @version $Revision: 1.10.2.2 $
 */
public class Config {
    public static String localProviderID;
    public static String localProviderApp;
    public static String alternativeLocalProviderApp;
    public static String providerDescriptorFactory;
    public static String xmlProviderDescriptorFile;
    public static String providerMetadataICF;
    public static String providerMetadataURL;
    public static String signingKeystore;
    public static String signingKeystorePass;
    public static String signingKeyAlias;
    public static String signingKeyPass;
    public static String signingKeyAlg;
    public static String signingKeyJAlg;
    public static String loginConfID;
    public static String userMetadataICF;
    public static String userMetadataURL;
    public static String userMetadataPrin;
    public static String userMetadataCred;
    public static boolean debug;
    public static boolean debugAuthn;
    public static boolean compoundAuthn;
    public static boolean xmlDebug;
    public static boolean timeDebug;
    public static String timesFile;
    public static int debugLevel;
    public static long debugFlushTimeout;
    public static boolean useFormPost;
    public static long issueInstantTolerance;
    public static long answerInstantTolerancePre;
    public static long answerInstantTolerancePost;
    public static long artifactAgeLimit;
    public static boolean checkSignatures;
    public static boolean isolatedSignatureVerification;
    public static String mainLoginPage;
    public static String loginPage;
    public static String autoLoginPage;
    public static String successfulRadiusAuthnPage;
    public static String successfulLibertyAuthnPage;
    public static String failedAuthnPage;
    public static String alreadyAuthnPage;
    public static String alreadyLibertyAuthnPage;
    public static String federationPage;
    public static String federationPageForRedirect;
    public static String logoutPage;
    public static String logoutPageForRedirect;
    public static String logoutDonePage;
    public static String consentPage;
    public static String ipManager;
    public static long accessTimeout;
    public static long urlAccessTimeout;
    public static String radiusServer;
    public static String radiusPort;
    public static String radiusAcctPort;
    public static String radiusSecret;
    public static boolean interactive;
    public static int L2RemotePort;
    public static int L2LocalPort;
    public static String authnCapabilitiesFile;


    /* Default init -- probably not useful */
    static {
	localProviderID = "localhost_SP";
	localProviderApp = "http://localhost:8080/sp";
        providerDescriptorFactory =
            "com.sun.liberty.XMLProviderDescriptorFactory";
        xmlProviderDescriptorFile = "providers.xml";
        providerMetadataICF = "com.sun.jndi.ldap.LdapCtxFactory";
	providerMetadataURL
	    = "ldap://localhost/ou=People,dc=Sahara,dc=SSO";
	signingKeystore = "./test/src/keystore";
	signingKeystorePass = "changeit";
	signingKeyAlias = "liberty";
	signingKeyPass = "changeit";
	signingKeyAlg = "http://www.w3.org/2000/09/xmldsig#rsa-sha1";
	signingKeyJAlg = "SHA1withRSA";
	userMetadataICF = "com.sun.jndi.ldap.LdapCtxFactory";
	userMetadataURL
	    = "ldap://localhost/ou=People,dc=Sahara,dc=SSO";
	debug = false;
	debugAuthn = false;
	compoundAuthn = false;
	xmlDebug = false;
	timeDebug = false;
        timesFile = "measurements.txt";
        debugLevel = 0;
        debugFlushTimeout = 60;
	useFormPost = false;
	issueInstantTolerance = 60000;
	answerInstantTolerancePre = 30000;
	answerInstantTolerancePost = 1000;
	artifactAgeLimit = 60000;
	checkSignatures = true;
	isolatedSignatureVerification = false;
        loginPage = "/login.jsp";
        federationPage = "/federate.jsp";
        federationPageForRedirect = "federate.jsp";
        logoutPage = "/logout.jsp";
        logoutPageForRedirect = "logout.jsp";
        logoutDonePage = "/logout-done.jsp";
        consentPage = "/consent.jsp";
	radiusServer = "localhost";
	radiusPort = "1812";
	radiusAcctPort = "1813";
	radiusSecret = "foobar";
	interactive =  false;
        L2RemotePort = 20000;
        L2LocalPort = 20002;
        authnCapabilitiesFile = "authn_capabilities.xml";
        accessTimeout = 30;
        urlAccessTimeout = 2;
    }

    /*
    public static init(ServletContext context) {
	providerMetadataICF = context.getInitParameter("providerMetadataICF");
	providerMetadataURL = context.getInitParameter("providerMetadataURL");
	signingKeystore = context.getInitParameter("signingKeystore");
	signingKeystorePass = context.getInitParameter("signingKeystorePass");
	signingKeyAlias = context.getInitParameter("signingKeyAlias");
	signingKeyPass = context.getInitParameter("signingKeyPass");
	signingKeyAlg = context.getInitParameter("signingKeyAlg");
	signingKeyJAlg = context.getInitParameter("signingKeyJAlg");
    }
     */

    /**
     * Initialize the parameters using the given properties, which should
     * have names corresponding to the field names of this object.
     */
    public static void init(Properties props) {
	localProviderID = props.getProperty("localProviderID");
	localProviderApp = props.getProperty("localProviderApp");
	alternativeLocalProviderApp = 
            props.getProperty("alternativeLocalProviderApp");
        providerDescriptorFactory =
            props.getProperty("providerDescriptorFactory");
        xmlProviderDescriptorFile =
            props.getProperty("xmlProviderDescriptorFile");
        providerMetadataICF = props.getProperty("providerMetadataICF");
	providerMetadataURL = props.getProperty("providerMetadataURL");
	signingKeystore = props.getProperty("signingKeystore");
	signingKeystorePass = props.getProperty("signingKeystorePass");
	signingKeyAlias = props.getProperty("signingKeyAlias");
	signingKeyPass = props.getProperty("signingKeyPass");
	signingKeyAlg = props.getProperty("signingKeyAlg");
	signingKeyJAlg = props.getProperty("signingKeyJAlg");
	loginConfID = props.getProperty("loginConfID");
	userMetadataICF = props.getProperty("userMetadataICF");
	userMetadataURL = props.getProperty("userMetadataURL");
	userMetadataPrin = props.getProperty("userMetadataPrin");
	userMetadataCred = props.getProperty("userMetadataCred");
	debug = Boolean.valueOf(props.getProperty("debug")).booleanValue();
	debugAuthn = Boolean.valueOf(props.getProperty("debugAuthn")).booleanValue();
	compoundAuthn = Boolean.valueOf(props.getProperty("compoundAuthn")).booleanValue();
	xmlDebug = Boolean.valueOf(props.getProperty("xmlDebug")).booleanValue();
	timeDebug = Boolean.valueOf(props.getProperty("timeDebug")).booleanValue();
	debugLevel = Integer.valueOf(props.getProperty("debugLevel")).intValue();
	debugFlushTimeout = Integer.valueOf(props.getProperty("debugFlushTimeout")).longValue();
	timesFile = props.getProperty("timesFile");
	useFormPost = Boolean.valueOf(props.getProperty("useFormPost")).booleanValue();
	issueInstantTolerance = Integer.valueOf(props.getProperty("issueInstantTolerance")).longValue();
	answerInstantTolerancePre = Integer.valueOf(props.getProperty("answerInstantTolerancePre")).longValue();
	answerInstantTolerancePost = Integer.valueOf(props.getProperty("answerInstantTolerancePost")).longValue();
	artifactAgeLimit = Integer.valueOf(props.getProperty("artifactAgeLimit")).longValue();
	checkSignatures = Boolean.valueOf(props.getProperty("checkSignatures")).booleanValue();
	isolatedSignatureVerification = Boolean.valueOf(props.getProperty("isolatedSignatureVerification")).booleanValue();
	mainLoginPage = props.getProperty("mainLoginPage");
	loginPage = props.getProperty("loginPage");
	autoLoginPage = props.getProperty("autoLoginPage");
	successfulRadiusAuthnPage = props.getProperty("successfulRadiusAuthnPage");
	successfulLibertyAuthnPage = props.getProperty("successfulLibertyAuthnPage");
	failedAuthnPage = props.getProperty("failedAuthnPage");
	alreadyAuthnPage = props.getProperty("alreadyAuthnPage");
	alreadyLibertyAuthnPage = props.getProperty("alreadyLibertyAuthnPage");
	federationPage = props.getProperty("federationPage");
	federationPageForRedirect = props.getProperty("federationPageForRedirect");
	logoutPage = props.getProperty("logoutPage");
	logoutPageForRedirect = props.getProperty("logoutPageForRedirect");
	logoutDonePage = props.getProperty("logoutDonePage");
	consentPage = props.getProperty("consentPage");
	ipManager = props.getProperty("ipManager");
	radiusServer = props.getProperty("radiusServer");
	radiusPort = props.getProperty("radiusPort");
	radiusAcctPort = props.getProperty("radiusAcctPort");
	radiusSecret = props.getProperty("radiusSecret");
	interactive = Boolean.valueOf(props.getProperty("interactive")).booleanValue();
	L2RemotePort = Integer.valueOf(props.getProperty("L2RemotePort")).intValue();
	L2LocalPort = Integer.valueOf(props.getProperty("L2LocalPort")).intValue();
        authnCapabilitiesFile = props.getProperty("authnCapabilitiesFile");
	accessTimeout = Integer.valueOf(props.getProperty("accessTimeout")).longValue();
	urlAccessTimeout = Integer.valueOf(props.getProperty("urlAccessTimeout")).longValue();
    }
}
