/*
 * $Id: LibertyAuthnContext.java,v 1.7.2.1 2002/09/17 01:52:22 jsergent Exp $
 * LibertyAuthnContext.java
 *
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.experimentalstuff.com/Technologies/IPL/download.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2002 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 */

package com.sun.liberty;
import com.sun.liberty.ac.LibertyAuthenticationContextStatement;
import java.util.Iterator;
import java.util.List;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * Data bindings for lib:AuthnContext element, used to specify
 * selected authentication context in an AuthnResponse.
 *
 * @author Jonathan Sergent
 * @version $Revision: 1.7.2.1 $
 */
public class LibertyAuthnContext extends XMLElement implements LibertyElement {
    /** Storage for contents of AuthnContextClassRef child */
    private String classRef = null;
    /** Storage for AuthenticationContextStatement child */
    private LibertyAuthenticationContextStatement statement = null;
    /** Storage for contents of AuthnContextStatementRef child */
    private String statementRef = null;

    /**
     * Parse XML element to populate fields.
     * XXX: Not yet implemented!
     * @param e Element of type lib:AuthnContext
     */
    public void parse(Element e) {
    }

    public Element getElement(Document doc) {
	Element a = doc.createElementNS(LIB_NS_URI, "lib:AuthnContext");
	addAttributesTo(a);
	addChildrenTo(a);
	return a;
    }

    /** 
     * Adds attributes from this class to the element.
     * (Note: no attributes from this class.)
     * @param s The element
     */
    protected void addAttributesTo(Element a) {
    }

    /** 
     * Adds child elements from this class to the element.
     * @param s The element
     */
    protected void addChildrenTo(Element a) {
	appendNewChildNSWithText(a, AUTHCTX_NS_URI, "AuthnContextClassRef", classRef);
	if (statement != null) {
	    a.appendChild(statement.getElement(a.getOwnerDocument()));
	}
	appendNewChildNSWithText(a, AUTHCTX_NS_URI, "AuthnContextStatementRef",
		statementRef);
    }

    /**
     * Returns contents of AuthnContextClassRef child, if present.
     * @return the URI in a string, or null if not present.
     */
    public String getClassRef() {
	return classRef;
    }

    /**
     * Sets or clears the contents of the AuthnContextClassRef child.
     * @param value the URI in a string, or null.
     */
    public void setClassRef(String value) {
	classRef = value;
    }

    /**
     * Returns the AuthenticationContextStatement child element, if present.
     * @return the element, or null if not present.
     */
    public LibertyAuthenticationContextStatement getStatement() {
	return statement;
    }

    /**
     * Sets or clears the AuthenticationContextStatement child element.
     * @param value the element, or null.
     */
    public void setStatement(LibertyAuthenticationContextStatement value) {
	statement = value;
    }

    /**
     * Returns contents of AuthnContextStatementRef child, if present.
     * @return the URI in a string, or null if not present.
     */
    public String getStatementRef() {
	return statementRef;
    }

    /**
     * Sets or clears the contents of the AuthnContextStatementRef child.
     * @param value the URI in a string, or null.
     */
    public void setStatementRefs(String value) {
	statementRef = value;
    }
}
