/*
 * $Id: LibertyAuthnResponse.java,v 1.8.2.2 2002/09/17 01:52:22 jsergent Exp $
 * LibertyAuthnResponse.java
 *
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.experimentalstuff.com/Technologies/IPL/download.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2002 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 */

package com.sun.liberty;
import com.sun.liberty.saml.SAMLPResponse;
import java.util.Iterator;
import java.util.List;
import java.util.LinkedList;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/**
 * Data bindings for the lib:AuthnResponse element.  This is the response
 * from the single sign-on protocol.  Contains the extensions made from
 * samlp:Response.
 * @author Jonathan Sergent
 * @version $Revision: 1.8.2.2 $
 */
public class LibertyAuthnResponse extends SAMLPResponse implements LibertyElement {
    /** Storage for RelayState value */
    String relayState;

    /**
     * Parse XML element to populate fields.
     * @param e lib:AuthnResponse element or extension thereof
     */
    public void parse(Element e) {
	setRelayState(e.getAttributeNS(null, "RelayState"));

	NodeList list = e.getChildNodes();
	for (int i = 0; i < list.getLength(); i++) {
	    if (list.item(i) instanceof org.w3c.dom.Text) { continue; }
	    Element child = (Element) list.item(i);
	    if (isElement(child, LIB_NS_URI, "Assertion")) {
		LibertyAssertion assertion = new LibertyAssertion();
		assertion.parse(child);
		List set = getAssertions();
		if (set == null) {
		    set = new LinkedList();
		}
		set.add(assertion);
		setAssertions(set);	
	    }
	}
	super.parse(e);
    }
    
    public Element getElement(Document doc) {
	Element r = doc.createElementNS(LIB_NS_URI, "lib:AuthnResponse");
	addAttributesTo(r);
	addChildrenTo(r);
	return r;
    }

    protected void addAttributesTo(Element s) {
	super.addAttributesTo(s);
    }

    protected void addChildrenTo(Element s) {
	appendNewChildNSWithText(s, LIB_NS_URI, "lib:RelayState", relayState);
	super.addChildrenTo(s);
    }

    /**
     * Returns value of RelayState field.  Used to pass back the contents
     * of the RelayState field from the AuthnRequest.
     * @return the string, or null if not present
     */
    public String getRelayState() {
	return relayState;
    }

    /**
     * Sets or clears value of RelayState field.  Used to pass back the
     * contents of the RelayState field from the AuthnRequest.
     * @param value the new state value, or null
     */
    public void setRelayState(String value) {
	relayState = value;
    }
}
