/*
 * $Id: LibertyElement.java,v 1.5.2.1 2002/09/17 01:52:22 jsergent Exp $
 * LibertyElement.java
 *
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.experimentalstuff.com/Technologies/IPL/download.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2002 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 */

package com.sun.liberty;

/**
 * Interface specifying constants for use in XMLElement subclasses that
 * implement elements from the Liberty core schema.  Includes namespace
 * identifier URIs.
 * @author Jonathan Sergent
 * @version $Revision: 1.5.2.1 $
 */
public interface LibertyElement {
    /** Base URI for Liberty namespaces */
    public static final String NS_URI = "http://projectliberty.org/";
    /** Base URI for Liberty schemas */
    public static final String SCHEMA_NS_URI = NS_URI + "schemas/";
    /** URI for Liberty core schema (from protocols-schemas doc) */
    public static final String LIB_NS_URI = SCHEMA_NS_URI + "core/2002/05";
    /** URI for Liberty authenticationn context schema */
    public static final String AUTHCTX_NS_URI = SCHEMA_NS_URI + "authctx/2002/05";
    
    /** Base URI for protocol profiles */
    public static final String PROFILE_NS_URI = NS_URI + "profiles/";
    /** URI for browser artifact single sign-on profile */
    public static final String BRWS_ART_PROFILE_URI
        = PROFILE_NS_URI + "brws-art";
    /** URI for browser post single sign-on profile */
    public static final String BRWS_POST_PROFILE_URI
        = PROFILE_NS_URI + "brws-post";
    /** URI for WML post single sign-on profile */
    public static final String WML_POST_PROFILE_URI
        = PROFILE_NS_URI + "wml-post";
    /** URI for Liberty-enabled Client and Proxy single sign-on profile */
    public static final String LECP_PROFILE_URI = PROFILE_NS_URI + "lecp";
    /** URI for HTTP-based IDP-initiated federation termination profile */
    public static final String FT_IDP_HTTP_PROFILE_URI
        = PROFILE_NS_URI + "fedterm-idp-http";
    /** URI for SOAP-based IDP-initiated federation termination profile */
    public static final String FT_IDP_SOAP_PROFILE_URI
        = PROFILE_NS_URI + "fedterm-idp-soap";
    /** URI for HTTP-based SP-initiated federation termination profile */
    public static final String FT_SP_HTTP_PROFILE_URI
        = PROFILE_NS_URI + "fedterm-sp-http";
    /** URI for SOAP-based SP-initiated federation termination profile */
    public static final String FT_SP_SOAP_PROFILE_URI
        = PROFILE_NS_URI + "fedterm-sp-soap";
    /** URI for HTTP-302-based IDP-initiated single logout profile */
    public static final String SL_IDP_HTTP_PROFILE_URI
        = PROFILE_NS_URI + "slo-idp-http";
    /** URI for HTTP-GET-based IDP-initiated single logout profile */
    public static final String SL_IDP_HTTP_GET_PROFILE_URI
        = PROFILE_NS_URI + "slo-idp-http-get";
    /** URI for SOAP-based IDP-initiated single logout profile */
    public static final String SL_IDP_SOAP_PROFILE_URI
        = PROFILE_NS_URI + "slo-idp-soap";
    /** URI for HTTP-based SP-initiated single logout profile */
    public static final String SL_SP_HTTP_PROFILE_URI
        = PROFILE_NS_URI + "slo-sp-http";
    /** URI for SOAP-based SP-initiated single logout profile */
    public static final String SL_SP_SOAP_PROFILE_URI
        = PROFILE_NS_URI + "slo-sp-soap";
    

    /*
     * Note that authentication context profiles are defined in their own
     * element...
     */
}
