/*
 * $Id: LibertyFederationTerminationNotification.java,v 1.6.2.2 2002/09/17 01:52:22 jsergent Exp $
 * LibertyFederationTerminationNotification.java
 *
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.experimentalstuff.com/Technologies/IPL/download.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2002 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 */

package com.sun.liberty;
import com.sun.liberty.saml.SAMLNameIdentifier;
import com.sun.liberty.saml.SAMLPAbstractRequest;
import java.util.Iterator;
import java.util.List;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/**
 * Data bindings for lib:FederationTerminationNotification message.
 * Used to inform an IDP or an SP of a request to terminate federation.
 * It is typically only useful to instantiate one of the subclasses of
 * this class that implements a particular Liberty profile.
 * @author Jonathan Sergent
 * @version $Revision: 1.6.2.2 $
 */
public class LibertyFederationTerminationNotification extends SAMLPAbstractRequest implements LibertyElement {
    /** Storage for ProviderID value */
    private String providerID;
    /** Storage for NameIdentifier value */
    private SAMLNameIdentifier nameIdentifier;

    public Element getElement(Document doc) {
	Element r = doc.createElementNS(LIB_NS_URI, "lib:FederationTerminationNotification");
	addAttributesTo(r);
	addChildrenTo(r);
	return r;
    }

    protected void addAttributesTo(Element r) {
	super.addAttributesTo(r);
    }

    protected void addChildrenTo(Element r) {
	appendNewChildNSWithText(r, LIB_NS_URI, "lib:ProviderID", providerID);
	if (nameIdentifier != null) {
	    r.appendChild(nameIdentifier.getElement(r.getOwnerDocument()));
	}
	super.addChildrenTo(r);
    }

    public void parse(Element e) {
	NodeList list = e.getChildNodes();
	for (int i = 0; i < list.getLength(); i++) {
	    if (list.item(i) instanceof org.w3c.dom.Text) { continue; }
	    Element child = (Element) list.item(i);
	    if (isElement(child, LIB_NS_URI, "ProviderID")) {
		setProviderID(child.getFirstChild().getNodeValue());
	    } else if (isElement(child, SAML_NS_URI, "NameIdentifier")) {
		SAMLNameIdentifier c = new SAMLNameIdentifier();
		c.parse(child);
		setNameIdentifier(c);
	    }
	}
	super.parse(e);
    }

    /**
     * Gets the value of the ProviderID child element.  Stores the provider
     * ID of the provider requesting termination.
     * @return String the provider ID
     */
    public String getProviderID() {
	return providerID;
    }

    /**
     * Sets the value of the ProviderID child element.  Stores the provider
     * ID of the provider requesting termination.
     * @param value the provider ID
     */
    public void setProviderID(String value) {
	providerID = value;
	setReceivedFrom(value);
    }

    /**
     * Returns the name identifier for the user to be terminated.
     * This is the name identifier that was provided by the provider
     * to whom the message is being sent.
     * @return the name identifier
     */
    public SAMLNameIdentifier getNameIdentifier() {
	return nameIdentifier;
    }

    /**
     * Sets the name identifier for the user to be terminated.
     * This is the name identifier that was provided by the provider
     * to whom the message is being sent.
     * @param value the name identifier
     */
    public void setNameIdentifier(SAMLNameIdentifier value) {
	nameIdentifier = value;
    }
}
