/*
 * $Id: LibertyLogoutNotification.java,v 1.4.2.2 2002/09/17 01:52:23 jsergent Exp $
 * LibertyLogoutNotification.java
 *
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.experimentalstuff.com/Technologies/IPL/download.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2002 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 */

package com.sun.liberty;
import com.sun.liberty.saml.SAMLNameIdentifier;
import com.sun.liberty.saml.SAMLPAbstractRequest;
import java.util.Iterator;
import java.util.List;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/**
 * Data bindings for lib:LogoutNotification element.  Used to request
 * single log-out.  If sent to an IDP, the IDP notifies each SP
 * that it has given an authentication assertion to using the profile
 * specified in the metadata of the SP.
 * 
 * @author Jonathan Sergent
 * @version $Revision: 1.4.2.2 $
 */
public class LibertyLogoutNotification extends SAMLPAbstractRequest implements LibertyElement {
    /** Storage for ProviderID */
    private String providerID;
    /** Storage for NameIdentifier */
    private SAMLNameIdentifier nameIdentifier;
    /** Storage for SessionIndex */
    private String sessionIndex;

    public Element getElement(Document doc) {
        Element r = doc.createElementNS(LIB_NS_URI, "lib:LogoutNotification");
        addAttributesTo(r);
        addChildrenTo(r);
        return r;
    }

    protected void addAttributesTo(Element r) {
        super.addAttributesTo(r);
    }

    protected void addChildrenTo(Element r) {
        appendNewChildNSWithText(r, LIB_NS_URI, "lib:ProviderID", providerID);
        appendNewChildNSWithText(r, LIB_NS_URI, "lib:SessionIndex", sessionIndex);
        if (nameIdentifier != null) {
            r.appendChild(nameIdentifier.getElement(r.getOwnerDocument()));
        }
        super.addChildrenTo(r);
    }

    public void parse(Element e) {
        NodeList list = e.getChildNodes();
        for (int i = 0; i < list.getLength(); i++) {
            if (list.item(i) instanceof org.w3c.dom.Text) { continue; }
            Element child = (Element) list.item(i);
            if (isElement(child, LIB_NS_URI, "ProviderID")) {
                setProviderID(child.getFirstChild().getNodeValue());
            } else if (isElement(child, LIB_NS_URI, "SessionIndex")) {
                setSessionIndex(child.getFirstChild().getNodeValue());
            } else if (isElement(child, SAML_NS_URI, "NameIdentifier")) {
                SAMLNameIdentifier c = new SAMLNameIdentifier();
                c.parse(child);
                setNameIdentifier(c);
            }
        }
        super.parse(e);
    }

    /**
     * Returns the value of the ProviderID child element.  Contains the
     * ID of the provider requesting logout.
     * @return the provider ID
     */
    public String getProviderID() {
        return providerID;
    }

    /**
     * Sets or clears the value of the ProviderID child element.  
     * Contains the ID of the provider requesting logout.
     * @param value the provider ID
     */
    public void setProviderID(String value) {
        providerID = value;
        setReceivedFrom(value);
    }

    /**
     * Gets the value of the SessionIndex child element.  Should be
     * the contents of the SessionIndex from the AuthnResponse sent
     * from the IDP to the SP to initiate the session being logged out.
     * @return The session index, or null if not present
     */
    public String getSessionIndex() {
        return sessionIndex;
    }

    /**
     * Sets or clears the value of the SessionIndex child element.  Should be
     * the contents of the SessionIndex from the AuthnResponse sent
     * from the IDP to the SP to initiate the session being logged out.
     * @param value The session index, or null
     */
    public void setSessionIndex(String value) {
        sessionIndex = value;
    }

    /**
     * Returns the name identifier of the user being logged out.  Must
     * be the name identifier provided by the provider who the message
     * is being sent to.
     * @return The name identifier
     */
    public SAMLNameIdentifier getNameIdentifier() {
        return nameIdentifier;
    }

    /**
     * Sets or clears the name identifier of the user being logged out.  
     * Must be the name identifier provided by the provider who the message
     * is being sent to.
     * @return The name identifier
     */
    public void setNameIdentifier(SAMLNameIdentifier value) {
        nameIdentifier = value;
    }
}
