/*
 * $Id: LibertyManager.java,v 1.1.2.2 2002/09/17 01:52:23 jsergent Exp $
 * LibertyManager.java
 *
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.experimentalstuff.com/Technologies/IPL/download.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2002 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 */

package com.sun.liberty;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

public class LibertyManager {
    static ProviderDescriptorFactory factory;
   
    static {
       factory = ProviderDescriptorFactory.getInstance();
    }

    /**
     * Returns a list of all trusted IDPs (identity 
     * providers).
     * 
     * @return an iterator to a list of strings, each
     *         containing the provider ID of an IDP.
     */
    public static java.util.Iterator getIDPList() {
	java.util.List out = new java.util.LinkedList();
	java.util.Iterator i = factory.getIDPList().iterator();
	while (i.hasNext()) {
	    IDPDescriptor idp = (IDPDescriptor) i.next();
	    out.add(idp.getProviderID());
	}
	return out.iterator();
    }

    /**
     * Returns a list of all trusted SPs (service 
     * providers).
     * 
     * @return an iterator to a list of strings, each
     *         containing the provider ID of an SP.
     */
    public static java.util.Iterator getSPList() {
	java.util.List out = new java.util.LinkedList();
	java.util.Iterator i = factory.getSPList().iterator();
	while (i.hasNext()) {
	    SPDescriptor sp = (SPDescriptor) i.next();
	    out.add(sp.getProviderID());
	}
	return out.iterator();
    }

    /**
     * Gets the federation status of a user with an IDP.
     * 
     * @param user The username obtained by calling 
     *             getRemoteUser() on a Liberty-authenticated 
     *             HttpServletRequest from the user.
     * @param provider The provider ID of the IDP.
     */
    public static boolean getIDPFederationStatus(
	    String user, String provider) {
	try {
	    LibertyUser lu = LibertyUser.find(user);
	    IDPDescriptor idp = factory.findIDP(provider);
	    return lu.getFederationStatus(idp);
	} catch (Exception e) {
	    return false;
	}
    }

    /**
     * Gets the federation status of a user with an SP.
     * 
     * @param user The username obtained by calling 
     *             getRemoteUser() on a Liberty-authenticated 
     *             HttpServletRequest from the user.
     * @param provider The provider ID of the SP.
     */
    public static boolean getSPFederationStatus(
	    String user, String provider) {
	try {
	    LibertyUser lu = LibertyUser.find(user);
	    SPDescriptor sp = factory.findSP(provider);
	    return lu.getFederationStatus(sp);
	} catch (Exception e) {
	    return false;
	}
    }

    private static LibertyUserSession getSession(HttpServletRequest request) {
	HttpSession ses = request.getSession(true);
	return (LibertyUserSession) ses.getAttribute("liberty.session");
    }

    /**
     * Gets a nonce for use in forms to be posted to well
     * known servlets.  Avoids cross site scripting type attacks.
     * 
     * @param request A request bound to the session for which a nonce
     *             is needed.
     * @return A string to be put in a hidden form field
     *         called "nonce".
     */
    public static String getNonce(HttpServletRequest request) {
	LibertyUserSession lus = getSession(request);
	lus.newFCNonce();
	if (lus == null) {
	    return null;
	} else {
	    return lus.getFCNonce();
	}
    }

    /**
     * Checks that the given nonce is the same as the last one
     * returned via getNonce, and invalidates it.
     * 
     * @param request A request bound to the session for which the nonce
     *             was received.
     * @param request A request bound to the session for which the nonce
     *             is to be checked.
     */
    public static boolean checkNonce(String nonce, HttpServletRequest request) {
	LibertyUserSession lus = getSession(request);
	if (lus == null) {
	    return false;
	} else {
	    String sesNonce = lus.getFCNonce();
	    lus.newFCNonce();
	    return sesNonce.equals(nonce);
	}
    }

    /**
     * Gets the ID of the provider discovered via the
     * introduction protocol.  If null, no provider was
     * discovered.  Can be passed to LoginServlet if not null.
     *
     * @param request A request bound to the session whose provider
     *             should be discovered.
     * @return the provider ID
     */
    public static String getIntroducedProvider(
	    javax.servlet.http.HttpServletRequest req) {
	// return getSession(req).getIntroducedProvider();
	return null;
    }
}
