/*
 * $Id: SimpleSAMLRequestProcessor.java,v 1.5.2.1 2002/09/17 01:52:27 jsergent Exp $
 * SimpleSAMLRequestProcessor.java
 *
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.experimentalstuff.com/Technologies/IPL/download.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2002 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 */

package com.sun.liberty;
import com.sun.liberty.saml.SAMLPRequest;
import com.sun.liberty.saml.SAMLPResponse;

/**
 * This implementation of SAMLRequestProcessor only supports artifact
 * processing -- if a request contains an artifact, the response will
 * contain the requested assertion.
 * 
 * @author Jonathan Sergent
 * @version $Revision: 1.5.2.1 $
 */

public class SimpleSAMLRequestProcessor implements SAMLRequestProcessor {    
    /**
     * Dereferences artifacts from request to fill in response.
     * If request is not for an artifact, fails.
     * Note that data is copied from the response that was registered,
     * since it is not necessarily a SAMLPResponse.  This means that any
     * data specific to extensions to samlp:Response will not be sent.
     * This is a protocol problem, not an API problem.
     * @param request the request.
     * @param response the response.
     */
    public RequestProcessorAction process(SAMLPRequest request,
					  SAMLPResponse response)
    {
	String artifact = request.getAssertionArtifact();
	byte[] binArtifact = null;
	try {
	    binArtifact = URLUtils.base64Decode(artifact);
	} catch (Exception e) {}

	String sb = request.getSignedBy();
	String intended = ArtifactManager.retrieveAudience(binArtifact);
	if (Config.checkSignatures && (intended == null || sb == null
				       || !sb.equals(intended))) {
	    if (Config.debug) {
		System.err.println("SAMLPRequest was not properly signed");
	    }
	    return RequestProcessorAction.FAILED;
	}

	if (artifact == null) {
	    if (Config.debug) {
		System.err.println("Got SAMLPRequest without artifact");
	    }
	    return RequestProcessorAction.FAILED;
	}

	LibertyArtifactMessage msg = null;
        msg = ArtifactManager.retrieveArtifact(binArtifact);
        if (Config.debug) { System.err.println("msg:" + msg); }

	if (msg == null) {
	    if (Config.debug) {
		System.err.println("Could not dereference artifact: " 
			+ artifact);
	    }
	    /* XXX: Check to see if this is what SAML says we should do */
	    return RequestProcessorAction.FAILED;
	}

	SAMLPResponse responseMsg = (SAMLPResponse) msg;

	/*
	 * XXX: This is sub-optimal. Would be nice if we had copy methods
	 * in our XMLElements.
	 */
	response.setStatus(responseMsg.getStatus());
	response.setAssertions(responseMsg.getAssertions());
	response.setResponseID(responseMsg.getResponseID());
	response.setInResponseTo(responseMsg.getInResponseTo());
	response.setMajorVersion(responseMsg.getMajorVersion());
	response.setMinorVersion(responseMsg.getMinorVersion());
	response.setIssueInstant(responseMsg.getIssueInstant());
	response.setRecipient(responseMsg.getRecipient());
	response.setSignedBy(responseMsg.getSignedBy());
	if (Config.debug) { System.err.println("Sending back assertion okay"); }
	return RequestProcessorAction.RETURN_RESPONSE;
    }
}
