/*
 * $Id: SoapUtils.java,v 1.6.2.1 2002/09/17 01:52:27 jsergent Exp $
 * SoapUtils.java
 *
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.experimentalstuff.com/Technologies/IPL/download.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2002 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): Ana Sanz Merino [University of California Berkeley, 2003]
 */

package com.sun.liberty;

import ucb.eecs.sahara.sso.DebugUtils;

import java.io.ByteArrayOutputStream;
import javax.xml.soap.SOAPMessage;
import javax.xml.soap.SOAPConnection;
import javax.xml.soap.SOAPConnectionFactory;
import javax.xml.soap.MessageFactory;
import javax.xml.transform.Source;
import javax.xml.transform.dom.DOMResult;
import javax.xml.transform.dom.DOMSource;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

/**
 * Utility functions for SOAP processing using SAAJ.
 *
 * @author Jonathan Sergent
 * @version $Revision: 1.6.2.1 $
 */
public class SoapUtils {
    static SOAPConnectionFactory connFactory;
    static MessageFactory msgFactory;

    static {
	try {
	    msgFactory = MessageFactory.newInstance();
	    connFactory = SOAPConnectionFactory.newInstance();
	} catch (Exception e) {
	    e.printStackTrace();
	}
    }

    /**
     * Gets the envelope element from a SOAP message
     * @param msg the SOAP message
     * @return the Envelope element
     */
    public static Element getEnvelopeElement(SOAPMessage msg) {
	Source src;
	try {
	    ByteArrayOutputStream os = new ByteArrayOutputStream();
	    msg.writeTo(os);
	    Document doc = XMLUtils.parse(os.toString());
	    return doc.getDocumentElement();
	} catch (Exception e) {
	    e.printStackTrace();
	    return null;
	}
    }

    private static Element getEnvelopeChild(SOAPMessage msg, String name) {
	Element env = getEnvelopeElement(msg);
	for (Node n = env.getFirstChild(); n != null; n = n.getNextSibling()) {
	    if (n instanceof Element) {
		Element e = (Element) n;
		if (XMLElement.isElement(e, XMLElement.SOAP_NS_URI, name)) {
		    return e;
		}
	    }
	}
	return null;
    }

    /**
     * Gets the body element from a SOAP message
     * @param msg the SOAP message
     * @return the Body element
     */
    public static Element getBodyElement(SOAPMessage msg) {
	Element e = getEnvelopeChild(msg, "Body");
	return e;
    }

    /**
     * Gets the first child element of the body element from a SOAP message
     * @param msg the SOAP message
     * @return the child element
     */
    public static Element getRequestElement(SOAPMessage msg) {
	Element body = getBodyElement(msg);
	for (Node n = body.getFirstChild(); n != null; n = n.getNextSibling()) {
	    if (n instanceof Element) {
		return (Element) n;
	    }
	}
	return null;
    }

    /**
     * Gets the header element from a SOAP message
     * @param msg the SOAP message
     * @return the Header element
     */
    public static Element getHeaderElement(SOAPMessage msg) {
	return getEnvelopeChild(msg, "Header");
    }

    /**
     * Add the Liberty message to the body of the SOAP message
     * @param msg the Liberty message
     * @param soap the SOAPMessage to be sent
     */
    public static void addToBody(LibertySoapMessage msg, SOAPMessage soap) {
	Element body = getBodyElement(soap);
	Document doc = body.getOwnerDocument();
	body.appendChild(((XMLElement) msg).getElement(doc));
	try {
	    soap.getSOAPPart().setContent(new DOMSource(doc));
	} catch (Exception e) {
	    e.printStackTrace();
	    throw new RuntimeException("setContent() failed--see console");
	}
    }

    /**
     * Sends the Liberty message over SOAP with an empty SOAP header
     * to a SOAP endpoint
     * @param msg the Liberty message
     * @param url the SOAP endpoint URL
     */
    public static SOAPMessage send(LibertySoapMessage msg, String url) {
	try {
	    SOAPConnection conn = connFactory.createConnection();
            DebugUtils.debugTime("SoapUtils", "SOAP Connection created",
                                 DebugUtils.LOW_LEVEL_FUNC);
	    SOAPMessage soap = msgFactory.createMessage();
            DebugUtils.debugTime("SoapUtils", "SOAP message created",
                                 DebugUtils.LOW_LEVEL_FUNC);
	    msg.addToBody(soap);
            DebugUtils.debugTime("SoapUtils", "Message content added",
                                 DebugUtils.LOW_LEVEL_FUNC);
	    if (Config.debug) {
		soap.writeTo(System.err);
	    }
	    return conn.call(soap, url);
	    /*
	     * Note that any SOAP faults will be indicated in the reply
	     * message and should be checked by the caller...
	     */
	} catch (Exception e) {
	    e.printStackTrace();
	    return null;
	}
    }
}
