/*
 * $Id: SAMLAdvice.java,v 1.7.2.1 2002/09/17 01:52:29 jsergent Exp $
 * SAMLAdvice.java
 *
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.experimentalstuff.com/Technologies/IPL/download.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2002 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 */

package com.sun.liberty.saml;
import com.sun.liberty.DummyXMLElement;
import java.util.Iterator;
import java.util.List;
import java.util.LinkedList;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/**
 * Data bindings for saml:Advice element.
 *
 * @author Jonathan Sergent
 * @version $Revision: 1.7.2.1 $
 */

public class SAMLAdvice extends SAMLElement {
    private List assertions = null;
    private List assertionIDReferences = null;
    private List other = null;


    public void parse(Element e) {
	NodeList list = e.getChildNodes();
	for (int j = 0; j < list.getLength(); j++) {
	    if (list.item(j) instanceof org.w3c.dom.Text) { continue; }
	    Element child = (Element) list.item(j);
	    if (isElement(child, SAML_NS_URI, "Assertion")) {
		SAMLAssertion c = new SAMLAssertion();
		c.parse(child);
		List set = getAssertions();
		if (set == null) {
		    set = new LinkedList();
		}
		set.add(c);
		setAssertions(set);
	    } else if (isElement(child, SAML_NS_URI, "AssertionIDReference")) {
		List set = getAssertionIDReferences();
		if (set == null) {
		    set = new LinkedList();
		}
		set.add(child.getFirstChild().getNodeValue());
		setAssertionIDReferences(set);
	    } else {
		DummyXMLElement c = new DummyXMLElement(child);
		List set = getOther();
		if (set == null) {
		    set = new LinkedList();
		}
		set.add(c);
		setOther(set);
	    }
	}
    }

    public Element getElement(Document doc) {
        Element a = doc.createElementNS(SAML_NS_URI, "saml:Advice");
        addChildrenTo(a);
        return a;
    }

    protected void addChildrenTo(Element a) {
        iterateAddChildren(a, assertions);
        iterateAddChildren(a, other);
	if (assertionIDReferences != null) {
	    Iterator i = assertionIDReferences.iterator();
	    while (i.hasNext()) {
	    	Element r;
		r = a.getOwnerDocument().createElementNS(SAML_NS_URI, "saml:AssertionIDReference");
		a.appendChild(r);
		r.appendChild(a.getOwnerDocument().createTextNode((String) i.next()));
	    }
	}
    }
    

    /**
     * Returns the list of assertions.  Each node in the list is
     * a SAMLAssertion.
     * @return the list, or null if not present
     */
    public List getAssertions() {
        return assertions;
    }

    /**
     * Sets or clears the list of assertions.  Each node in the list
     * must be a SAMLAssertion or subclass thereof.
     * @param value the new list, or null
     */
    public void setAssertions(List value) {
        assertions = value;
    }

    /**
     * Returns the list of AssertionIDReference element values.  Each
     * node in the list is the value of an AssertionIDReference element.
     * @return the list, or null if not present
     */
    public List getAssertionIDReferences() {
        return assertionIDReferences;
    }

    /**
     * Sets or clears the list of AssertionIDReference element values.  Each
     * node in the list is the value of an AssertionIDReference element.
     * @return the list, or null if not present
     */
    public void setAssertionIDReferences(List value) {
        assertionIDReferences = value;
    }

    /**
     * Gets the list of "other" elements.  Each node in the list is an
     * XMLElement.
     * @return the list, or null if not present
     */
    public List getOther() {
        return other;
    }

    /**
     * Sets or clears the list of "other" elements.  Each node in the list
     * must be an XMLElement.
     * @return the list, or null if not present
     */
    public void setOther(List value) {
        other = value;
    }
}
