/*
 * $Id: SAMLAuthenticationStatement.java,v 1.5.2.1 2002/09/17 01:52:30 jsergent Exp $
 * SAMLAuthenticationStatement.java
 *
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.experimentalstuff.com/Technologies/IPL/download.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2002 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 */

package com.sun.liberty.saml;
import java.util.Date;
import java.util.List;
import java.util.LinkedList;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/**
 * Data bindings for saml:AuthenticationStatement element.  Get and set
 * methods correspond to child elements and attributes in the XML schema
 * for this element.
 *
 * @author Jonathan Sergent
 * @version $Revision: 1.5.2.1 $
 */

public class SAMLAuthenticationStatement extends SAMLSubjectStatement {
    String authenticationMethod = null;
    Date authenticationInstant = null;
    SAMLSubjectLocality subjectLocality = null;
    List authorityBindings = null;


    public void parse(Element e) {
	super.parse(e);
	setAuthenticationMethod(e.getAttributeNS(null, "AuthenticationMethod"));
	setAuthenticationInstant(parseDate(e.getAttributeNS(null, "AuthenticationInstant")));
	NodeList list = e.getChildNodes();
	for (int j = 0; j < list.getLength(); j++) {
	    if (list.item(j) instanceof org.w3c.dom.Text) { continue; }
	    Element child = (Element) list.item(j);
	    if (isElement(child, SAML_NS_URI, "SubjectLocality")) {
		SAMLSubjectLocality c = new SAMLSubjectLocality();
		c.parse(child);
		setSubjectLocality(c);
	    } else if (isElement(child, SAML_NS_URI, "AuthorityBinding")) {
		SAMLAuthorityBinding c = new SAMLAuthorityBinding();
		c.parse(child);
		List set = getAuthorityBindings();
		if (set == null) {
		    set = new LinkedList();
		}
		set.add(c);
		setAuthorityBindings(set);
	    }
	}
    }

    public Element getElement(Document doc) {
	Element s;
	s = doc.createElementNS(SAML_NS_URI, SAML_NS_ID + ":AuthenticationStatement");
	addAttributesTo(s);
	addChildrenTo(s);
	return s;
    }

    protected void addChildrenTo(Element s) {
	super.addChildrenTo(s);
	if (subjectLocality != null) {
	    s.appendChild(subjectLocality.getElement(s.getOwnerDocument()));
	}
	iterateAddChildren(s, authorityBindings);
    }

    protected void addAttributesTo(Element s) {
	if (authenticationMethod != null) {
	    s.setAttributeNS(null, "AuthenticationMethod", authenticationMethod);
	}
	if (authenticationInstant != null) {
	    s.setAttributeNS(null, "AuthenticationInstant",
		    formatDate(authenticationInstant));
	}
    }


    public String getAuthenticationMethod() {
	return authenticationMethod;
    }

    public void setAuthenticationMethod(String value) {
	authenticationMethod = value;
    }

    public Date getAuthenticationInstant() {
	return authenticationInstant;
    }

    public void setAuthenticationInstant(Date value) {
	authenticationInstant = value;
    }

    public SAMLSubjectLocality getSubjectLocality() {
	return subjectLocality;
    }

    public void setSubjectLocality(SAMLSubjectLocality value) {
	subjectLocality = value;
    }

    public List getAuthorityBindings() {
	return authorityBindings;
    }

    public void setAuthorityBindings(List value) {
	authorityBindings = value;
    }
}
