/*
 * $Id: SAMLPAbstractResponse.java,v 1.7.2.1 2002/09/17 01:52:31 jsergent Exp $
 * SAMLPAbstractResponse.java
 *
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.experimentalstuff.com/Technologies/IPL/download.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2002 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 */

package com.sun.liberty.saml;
import com.sun.liberty.XMLUtils;
import java.util.Iterator;
import java.util.Date;
import java.util.List;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/**
 * Data bindings for the samlp:AbstractResponseType type.  Get and set
 * methods correspond to child elements and attributes in the XML schema
 * for this element.
 *
 * @author Jonathan Sergent
 * @version $Revision: 1.7.2.1 $
 */

public abstract class SAMLPAbstractResponse extends SAMLElement {
    private String responseID = generateIDType();
    private String inResponseTo = null;
    private int majorVersion = 1;
    private int minorVersion = 0;
    private Date issueInstant = new Date();
    private String recipient = null;
    private String signedBy = null;
    private String receivedFrom = null;

    public void parse(Element e) {
	setResponseID(e.getAttributeNS(null, "ResponseID"));
	setInResponseTo(e.getAttributeNS(null, "InResponseTo"));
	setMajorVersion(Integer.valueOf(e.getAttributeNS(null, "MajorVersion")).intValue());
	setMinorVersion(Integer.valueOf(e.getAttributeNS(null, "MinorVersion")).intValue());
	setIssueInstant(parseDate(e.getAttributeNS(null, "IssueInstant")));
	setRecipient(e.getAttributeNS(null, "Recipient"));

	Element signature = null;
	NodeList list = e.getChildNodes();
	for (int i = 0; i < list.getLength(); i++) {
	    if (list.item(i) instanceof org.w3c.dom.Text) { continue; }
	    Element child = (Element) list.item(i);
	    if (isElement(child, DSIG_NS_URI, "Signature")) {
		signature = child;
	    }
	}

	if (signature != null) {
	    setSignedBy(XMLUtils.verify(signature, receivedFrom) ? receivedFrom : null);
	}
    }

    protected void addAttributesTo(Element s) {
	s.setAttributeNS(null, "ResponseID", responseID);
	s.setAttributeNS(null, "InResponseTo", inResponseTo);
	s.setAttributeNS(null, "MajorVersion", new Integer(majorVersion).toString());
	s.setAttributeNS(null, "MinorVersion", new Integer(minorVersion).toString());
	s.setAttributeNS(null, "IssueInstant", formatDate(issueInstant));
	s.setAttributeNS(null, "Recipient", recipient);
    }

    protected void addChildrenTo(Element s) {
	if (signedBy != null) {
	    XMLUtils.sign(s);
	}
    }

    public String getResponseID() {
	return responseID;
    }

    public void setResponseID(String value) {
	responseID = value;
    }

    public String getInResponseTo() {
	return inResponseTo;
    }

    public void setInResponseTo(String value) {
	inResponseTo = value;
    }

    public int getMajorVersion() {
	return majorVersion;
    }

    public void setMajorVersion(int value) {
	majorVersion = value;
    }

    public int getMinorVersion() {
	return minorVersion;
    }

    public void setMinorVersion(int value) {
	minorVersion = value;
    }

    public Date getIssueInstant() {
	return issueInstant;
    }

    public void setIssueInstant(Date value) {
	issueInstant = value;
    }

    public String getRecipient() {
	return recipient;
    }

    public void setRecipient(String value) {
	recipient = value;
    }

    public String getSignedBy() {
	return signedBy;
    }

    public void setSignedBy(String value) {
	signedBy = value;
    }

    public void setReceivedFrom(String value) {
	receivedFrom = value;
    }
}
