/*
 * $Id: SAMLPRequest.java,v 1.8.2.2 2002/09/17 01:52:31 jsergent Exp $
 * SAMLPRequest.java
 *
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.experimentalstuff.com/Technologies/IPL/download.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2002 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 */

package com.sun.liberty.saml;
import com.sun.liberty.ArtifactManager;
import com.sun.liberty.Config;
import com.sun.liberty.URLUtils;
import java.util.Iterator;
import java.util.Date;
import java.util.List;
import java.util.LinkedList;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/**
 * Data bindings for the samlp:Request element.  Get and set methods
 * correspond to child elements and attributes in the XML schema for this
 * element.
 *
 * @author Jonathan Sergent
 * @version $Revision: 1.8.2.2 $
 */

public class SAMLPRequest extends SAMLPAbstractRequest {
    /* SAMLPQuery query; */
    String assertionIDReference;
    String assertionArtifact;

    public void parse(Element e) {
        if (Config.debug) {
            System.out.println("p: e:" + e);
            org.w3c.dom.NamedNodeMap m = e.getAttributes();
            for (int i = 0; i < m.getLength(); i++) {
                System.out.println("p: name:" + m.item(i).getNodeName() + " value:" + m.item(i).getNodeValue());
            }
        }

	NodeList list = e.getChildNodes();
	for (int i = 0; i < list.getLength(); i++) {
	    if (list.item(i) instanceof org.w3c.dom.Text) { continue; }
	    Element child = (Element) list.item(i);
	    /*
	    if (isElement(child, SAMLP_NS_URI, "Query")) {
		SAMLPQuery query = new SAMLPQuery();
		query.parse(child);
		setQuery(query);
	    } else if (isElement(child, SAMLP_NS_URI, "SubjectQuery")) {
		SAMLPQuery query = new SAMLPSubjectQuery();
		query.parse(child);
		setQuery(query);
	    } else if (isElement(child, SAMLP_NS_URI, "AuthenticationQuery")) {
		SAMLPQuery query = new SAMLPAuthenticationQuery();
		query.parse(child);
		setQuery(query);
	    } else if (isElement(child, SAMLP_NS_URI, "AttributeQuery")) {
		SAMLPQuery query = new SAMLPAttributeQuery();
		query.parse(child);
		setQuery(query);
	    } else if (isElement(child, SAMLP_NS_URI, "AuthorizationDecisionQuery")) {
		SAMLPQuery query = new SAMLPAuthorizationDecisionQuery();
		query.parse(child);
		setQuery(query);
	    } else */
	    if (isElement(child, SAML_NS_URI, "AssertionIDReference")) {
		if (Config.debug) {
		    System.err.println("got assertion ID reference");
		}
		setAssertionIDReference(child.getFirstChild().getNodeValue());
	    } else if (isElement(child, SAMLP_NS_URI, "AssertionArtifact")) {
		if (Config.debug) {
		    System.err.println("got assertion artifact");
		}
		setAssertionArtifact(child.getFirstChild().getNodeValue());
		/*
		 * This is kind of a hack -- look at whom we gave this artifact
		 * to in order to determine which signature to check against.
		 */
		try {
		    setReceivedFrom(ArtifactManager.retrieveAudience(URLUtils.base64Decode(assertionArtifact)));
		} catch (Exception ex) {
		    ex.printStackTrace();
		}
	    } else {
		if (Config.debug) {
		    System.err.println("got unknown child: " + child.getNodeName());
		}
	    }
	}
	super.parse(e);
    }

    public Element getElement(Document doc) {
	Element r;
	r = doc.createElementNS(SAMLP_NS_URI, SAMLP_NS_ID + ":Request");
	addAttributesTo(r);
	addChildrenTo(r);
	return r;
    }

    protected void addAttributesTo(Element s) {
	super.addAttributesTo(s);
    }

    protected void addChildrenTo(Element s) {
	/*
	s.appendChild(query.getElement(s.getOwnerDocument()));
	*/
	appendNewChildNSWithText(s, SAML_NS_URI, "saml:AssertionIDReference",
		assertionIDReference);
	appendNewChildNSWithText(s, SAMLP_NS_URI, "samlp:AssertionArtifact",
		assertionArtifact);
	super.addChildrenTo(s);
    }

    /*
    public SAMLQuery getQuery() {
	return query;
    }

    public void setQuery(SAMLQuery value) {
	query = value;
    }
    */

    public String getAssertionIDReference() {
	return assertionIDReference;
    }

    public void setAssertionIDReference(String value) {
	assertionIDReference = value;
    }

    public String getAssertionArtifact() {
	return assertionArtifact;
    }

    public void setAssertionArtifact(String value) {
	if (Config.debug) {
	    System.err.println("setAssertionArtifact value:" + value);
	}
	assertionArtifact = value;
    }
}
