/*
 * $Id: SAMLPResponse.java,v 1.9.2.1 2002/09/17 01:52:31 jsergent Exp $
 * SAMLPResponse.java
 *
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.experimentalstuff.com/Technologies/IPL/download.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2002 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 */

package com.sun.liberty.saml;
import com.sun.liberty.LibertyAssertion;
import com.sun.liberty.LibertyElement;
import java.util.Iterator;
import java.util.Date;
import java.util.List;
import java.util.LinkedList;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/**
 * Data bindings for the samlp:Response element.  Get and set methods
 * correspond to child elements and attributes in the XML schema for this
 * element.
 *
 * @author Jonathan Sergent
 * @version $Revision: 1.9.2.1 $
 */

public class SAMLPResponse extends SAMLPAbstractResponse {
    SAMLPStatus status;
    List assertions;

    public void parse(Element e) {
	NodeList list = e.getChildNodes();
	for (int i = 0; i < list.getLength(); i++) {
	    if (list.item(i) instanceof org.w3c.dom.Text) { continue; }
	    Element child = (Element) list.item(i);
	    SAMLAssertion assertion = null;
	    if (isElement(child, SAMLP_NS_URI, "Status")) {
		SAMLPStatus status = new SAMLPStatus();
		status.parse(child);
		setStatus(status);
	    } else if (isElement(child, SAML_NS_URI, "Assertion")) {
		String type = child.getAttributeNS(XSI_NS_URI, "type");
		if (type.equals("lib:AssertionType")) {
		    assertion = new LibertyAssertion();
		} else {
		    assertion = new SAMLAssertion();
		}
	    } else if (isElement(child, LibertyElement.LIB_NS_URI, "Assertion")) {
		assertion = new LibertyAssertion();
	    }

	    if (assertion != null) {
		assertion.parse(child);
		List assertions = getAssertions();
		if (assertions == null) {
		    assertions = new LinkedList();
		}
		assertions.add(assertion);
		setAssertions(assertions);
	    }
	}
	super.parse(e);
    }

    public Element getElement(Document doc) {
	Element r;
	r = doc.createElementNS(SAMLP_NS_URI, SAMLP_NS_ID + ":Response");
	addAttributesTo(r);
	addChildrenTo(r);
	return r;
    }

    protected void addAttributesTo(Element s) {
	super.addAttributesTo(s);
    }

    protected void addChildrenTo(Element s) {
	iterateAddChildren(s, assertions);
	if (status != null) {
	    s.appendChild(status.getElement(s.getOwnerDocument()));
	}
	super.addChildrenTo(s);
    }

    public List getAssertions() {
	return assertions;
    }

    public void setAssertions(List value) {
	assertions = value;
    }

    public SAMLPStatus getStatus() {
	return status;
    }

    public void setStatus(SAMLPStatus value) {
	status = value;
    }
}
