/*
 * $Id: SAMLPStatus.java,v 1.6.2.1 2002/09/17 01:52:32 jsergent Exp $
 * SAMLStatus.java
 *
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.experimentalstuff.com/Technologies/IPL/download.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2002 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 */

package com.sun.liberty.saml;
import java.util.List;
import java.util.LinkedList;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/**
 * Data bindings for samlp:Status element.  Get and set methods correspond
 * to child elements and attributes in the XML schema for this element.
 * 
 * @author Jonathan Sergent
 * @version $Revision: 1.6.2.1 $
 */
public class SAMLPStatus extends SAMLElement {
    private String statusCode = null;
    private List statusMessages = null;
    private String statusDetail = null;


    public void parse(Element e) {
	NodeList list = e.getChildNodes();
	for (int j = 0; j < list.getLength(); j++) {
	    if (list.item(j) instanceof org.w3c.dom.Text) { continue; }
	    Element child = (Element) list.item(j);
	    if (isElement(child, SAMLP_NS_URI, "StatusCode")) {
		setStatusCode(child.getAttributeNS(null, "Value"));
	    } else if (isElement(child, SAMLP_NS_URI, "StatusMessage")) {
		String msg = child.getFirstChild().getNodeValue();
		List set = getStatusMessages();
		if (set == null) {
		    set = new LinkedList();
		}
		set.add(msg);
		setStatusMessages(set);
	    } else if (isElement(child, SAMLP_NS_URI, "StatusDetail")) {
		setStatusCode(child.getFirstChild().getNodeValue());
	    }
	}
    }

    
    public Element getElement(Document doc) {
	Element s = doc.createElementNS(SAMLP_NS_URI, "samlp:Status");
	addChildrenTo(s);
	return s;
    }

    protected void addChildrenTo(Element s) {
	/* FIXME: Should have SAMLPStatusCode children... */
	if (statusCode != null && !statusCode.equals("")) {
            Document doc = s.getOwnerDocument();
            Element c = doc.createElementNS(SAMLP_NS_URI, "samlp:StatusCode");
            s.appendChild(c);
	    c.setAttributeNS(null, "Value", statusCode);
	}
	iterateAddNewChildrenNSWithText(s, SAMLP_NS_URI, "samlp:StatusMessage",
				 statusMessages);
	appendNewChildNSWithText(s, SAMLP_NS_URI, "samlp:StatusDetail",
			  statusDetail);
    }

    public String getStatusCode() {
	return statusCode;
    }

    public void setStatusCode(String value) {
	statusCode = value;
    }

    public List getStatusMessages() {
	return statusMessages;
    }

    public void setStatusMessages(List value) {
	statusMessages = value;
    }

    public String getStatusDetail() {
	return statusDetail;
    }

    public void setStatusDetail(String value) {
	statusDetail = value;
    }
}
