/*
 * $Id: FederationServlet.java,v 1.1.2.2 2002/09/17 01:52:33 jsergent Exp $
 * FederationServlet.java
 *
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.experimentalstuff.com/Technologies/IPL/download.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2002 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 */

package com.sun.liberty.servlets;
import com.sun.liberty.Config;
import com.sun.liberty.IDPDescriptor;
import com.sun.liberty.LibertyProfile;
import com.sun.liberty.LibertyUser;
import com.sun.liberty.LibertyURLAuthnRequest;
import com.sun.liberty.LibertyUserSession;
import com.sun.liberty.ProviderDescriptorFactory;
import com.sun.liberty.SPDescriptor;
import java.io.IOException;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

/**
 *
 *
 * @author Jonathan Sergent
 * @version $Revision: 1.1.2.2 $
 */

public class FederationServlet extends HttpServlet {
    static ProviderDescriptorFactory dir;

    static {
	try {
	    dir = ProviderDescriptorFactory.getInstance();
	} catch (Exception e) {
	    e.printStackTrace();
	}
    }

    /**
     * Utility function to locate user's LibertyUserSession object from the
     * HttpSession.  If none is found, a new one is created.
     */
    private static LibertyUserSession getLibertySession(HttpServletRequest req) {
        HttpSession ses = req.getSession(true);
        LibertyUserSession lus;
        lus = (LibertyUserSession) ses.getAttribute("liberty.session");
        if (lus == null) {
            lus = new LibertyUserSession();
            ses.setAttribute("liberty.session", lus);
        }
	return lus;
    }

    /**
     * Implements standard servlet doGet interface.  Handles incoming
     * FederationTerminationNotifications that are URL-encoded.  SOAP
     * notifications come into the SoapServlet, not into here.
     */
    public void doPost(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
	LibertyUserSession lus = getLibertySession(req);
	String nonce = req.getParameter("nonce");

	if (nonce == null || !(nonce.equals(lus.getFCNonce()))) {
	    resp.sendRedirect(Config.federationPageForRedirect);
	    return;
	}

	lus.newFCNonce();
	LibertyUser lu = lus.getUser();
	if (lu == null) {
	    throw new ServletException("Current user is null");
	}
	String id = req.getParameter("_liberty_id");
	IDPDescriptor idp = dir.findIDP(id);
	if (idp == null) {
	    throw new ServletException("Cannot find IDP");
	}
	LibertyURLAuthnRequest ar = new LibertyURLAuthnRequest();
	ar.setProviderID(Config.localProviderID);
	if (((SPDescriptor) dir.findLocal()).getAuthnRequestsSigned()) {
	    ar.setSignedBy(Config.localProviderID);
	}
	ar.setFederate(true);
	ar.setIsPassive(false);
	ar.setForceAuthn(false);
	if (Config.useFormPost) {
	    ar.setProtocolProfile(LibertyProfile.BRWS_POST);
	}
	String url;
	try {
	    url = idp.getSingleSignOnURL() + "?" + ar.getQuery();
	} catch (Exception e) {
	    throw new ServletException("getQuery failed", e);
	}
	lus.setPendingAuthnRequest(ar);
	lus.setPendingRelayURL(Config.federationPageForRedirect);
	lus.setPendingIDP(id);
	/* Assertion servlet will do the actual federation. */
	resp.sendRedirect(url);
	return;
    }
}
