/*
 * $Id: IDPLogoutServlet.java,v 1.5.2.2 2002/09/17 01:52:34 jsergent Exp $
 * IDPLogoutServlet.java
 *
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.experimentalstuff.com/Technologies/IPL/download.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2002 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 */

package com.sun.liberty.servlets;
import com.sun.liberty.Config;
import com.sun.liberty.LibertyLogoutNotification;
import com.sun.liberty.LibertyURLLogoutNotification;
import com.sun.liberty.LibertyURLMessage;
import com.sun.liberty.LibertySoapLogoutNotification;
import com.sun.liberty.LibertySoapMessage;
import com.sun.liberty.LibertyProfile;
import com.sun.liberty.LibertyUser;
import com.sun.liberty.LibertyUserSession;
import com.sun.liberty.ProviderDescriptor;
import com.sun.liberty.ProviderDescriptorFactory;
import com.sun.liberty.SoapUtils;
import com.sun.liberty.SPDescriptor;
import com.sun.liberty.saml.SAMLNameIdentifier;
import java.io.IOException;
import java.util.Iterator;
import java.util.Set;
import javax.servlet.RequestDispatcher;
import javax.servlet.ServletConfig;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

/**
 * Executes IDP logout by sending necessary messages to all relying SPs
 * and then redirecting the user agent.
 */

public class IDPLogoutServlet extends HttpServlet {
    public void service(HttpServletRequest req, HttpServletResponse resp)
                       throws ServletException {
        // Standard protected resource garbage copied from protected.jsp
        // (Should be in a servlet filter, of course...)
        HttpSession ses = req.getSession(true);
        LibertyUserSession lus;
        lus = (LibertyUserSession) ses.getAttribute("liberty.session");
        if (lus == null) {
            lus = new LibertyUserSession();
            ses.setAttribute("liberty.session", lus);
        }
        LibertyUser lu = lus.getUser();
        ServletConfig cfg = getServletConfig();
        ServletContext ctx = cfg.getServletContext();
        if (lu == null) {
            RequestDispatcher disp;
            disp = ctx.getNamedDispatcher("LoginServlet");
            try {
                disp.forward(req, resp);
            } catch (IOException e) {
                throw new ServletException("Cannot forward", e);
            }
            lus = (LibertyUserSession) ses.getAttribute("liberty.session");
            if (lus != null) {
                lu = lus.getUser();
            }
        }
        if (lus == null || lu == null) {
            return;
        }
        ProviderDescriptorFactory fac = ProviderDescriptorFactory.getInstance();
        ProviderDescriptor local = fac.findLocal();
        String nonce = req.getParameter("nonce");
        if (lus.getPendingSLO() || (nonce != null
				    && nonce.equals(lus.getFCNonce()))) {
            Set sps = lus.getRelyingSPList();
	    if (sps != null) {
		Iterator i = sps.iterator();
		while (i.hasNext()) {
		    SPDescriptor sp = (SPDescriptor) i.next();
		    LibertyProfile profile = sp.getSingleLogoutProtocolProfile();
		    if (Config.debug) {
                        System.err.println("in FC IDP logout:");
                        System.err.println("sp:" + sp.getProviderID());
			System.err.println("profile:" + profile.toString());
		    }
		    LibertyLogoutNotification ln;
		    if (profile == LibertyProfile.SL_IDP_SOAP) {
			ln = new LibertySoapLogoutNotification();
		    } else {
			ln = new LibertyURLLogoutNotification();
		    }
		    ln.setProviderID(Config.localProviderID);
		    SAMLNameIdentifier nameid = new SAMLNameIdentifier();
		    try {
			nameid = lu.getNameProvidedBy(sp);
		    } catch (Exception e) {
			throw new ServletException("Can't look up name", e);
		    }
		    ln.setNameIdentifier(nameid);
		    ln.setSignedBy(Config.localProviderID);
		    if (profile == LibertyProfile.SL_IDP_SOAP) {
			SoapUtils.send((LibertySoapMessage) ln,
				       sp.getSoapEndpoint().toString());
		    } else {
			String url;
			try {
			    url = (sp.getSingleLogoutURL() + "?" + 
				   ((LibertyURLMessage) ln).getQuery());
			} catch (Exception e) {
			    throw new ServletException("Cannot format URL", e);
			}
			if (profile == LibertyProfile.SL_IDP_HTTP_GET) {
			    lus.addPendingSLOImage(url);
			} else if (profile == LibertyProfile.SL_IDP_HTTP) {
			    lus.addPendingSLORedirect(url);
			} else {
			    throw new ServletException("Unrecognized SLO profile");
			}
		    }
		}
	        sps.clear();
	    }
	    lus.setPendingSLO(true);
	    RequestDispatcher disp;
	    if (Config.debug) { System.err.println("On to completion"); }
	    disp = ctx.getNamedDispatcher("IDPLogoutCompletionServlet");
	    try {
		disp.forward(req, resp);
	    } catch (IOException e) {
		throw new ServletException("Cannot forward", e);
	    }
	    return;
        } else {
	    if (Config.debug) {
                System.err.println("Back to Logout Page!"); }
            try {
                ctx.getRequestDispatcher(Config.logoutPage).forward(req, resp);
            } catch (IOException e) {
                throw new ServletException("Cannot forward", e);
            }
            
            return;
        }
    }
}
