/*
 * $Id: LoginServlet.java,v 1.10.2.2 2002/09/17 01:52:34 jsergent Exp $
 * LoginRequest.java
 *
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.experimentalstuff.com/Technologies/IPL/download.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2002 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): Ana Sanz Merino [University of California Berkeley, 2003]
 */

package com.sun.liberty.servlets;

import ucb.eecs.sahara.sso.DebugUtils;

import ucb.eecs.sahara.sso.anp.ANPAuthnCapabilitiesManager;
import ucb.eecs.sahara.sso.anp.ANPRequestProcessor;
import ucb.eecs.sahara.sso.anp.SimpleANPRequestProcessor;
import ucb.eecs.sahara.sso.anp.ANPRequestProcResult;
import ucb.eecs.sahara.sso.anp.ANPRequest;
import ucb.eecs.sahara.sso.anp.ANPResponse;
import ucb.eecs.sahara.sso.anp.ANPRequestProcessorAction;
import ucb.eecs.sahara.sso.sp.AuthnControlSession;

import ucb.eecs.sahara.sso.authn.FirewallController;
import ucb.eecs.sahara.sso.authn.SessionRadiusAuthenticator;
import ucb.eecs.sahara.sso.authn.SessionLocalAuthenticator;
import ucb.eecs.sahara.sso.authn.LibertyAuthenticator;
import ucb.eecs.sahara.sso.authn.AuthenticatorResult;
import ucb.eecs.sahara.sso.authn.SimpleL2Authenticator;

import com.sun.liberty.Config;
import com.sun.liberty.IDPDescriptor;
import com.sun.liberty.LibertyProfile;
import com.sun.liberty.LibertyUser;
import com.sun.liberty.LibertyURLAuthnRequest;
import com.sun.liberty.LibertyUserSession;
import com.sun.liberty.ProviderDescriptor;
import com.sun.liberty.ProviderDescriptorFactory;
import com.sun.liberty.XMLUtils;
import com.sun.liberty.URLUtils;

import javax.security.auth.Subject;
import javax.security.auth.callback.Callback;
import javax.security.auth.callback.CallbackHandler;
import javax.security.auth.callback.NameCallback;
import javax.security.auth.callback.PasswordCallback;
import javax.security.auth.callback.UnsupportedCallbackException;
import javax.security.auth.login.LoginContext;
import javax.security.auth.login.LoginException;
import javax.servlet.RequestDispatcher;
import javax.servlet.ServletConfig;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequestWrapper;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import javax.servlet.http.Cookie;

import java.util.Enumeration;
import java.util.StringTokenizer;

import java.io.Writer;
import org.w3c.dom.Document;

import net.sourceforge.jradiusclient.RadiusClient;


/**
 * Servlet to authenticate a user at this provider.  User may be authenticated
 * via a local login or via a Liberty single sign-on exchange with another IDP.
 * Establishes the user's identity by setting the LibertyUser stored in the
 * LibertyUserSession, which is stored in the user's HTTP session in the
 * "liberty.session" attribute.
 *
 * Other servlets or JSPs should forward the user to this servlet before sending
 * the resource; this servlet should not be bound to its own URL.  This servlet
 * will not respond if authentication is successful, allowing access to the
 * resource to proceed as normal.
 * 
 * @author Jonathan Sergent
 * @version $Revision: 1.10.2.2 $
 */

public class LoginServlet extends HttpServlet {
    public static final String LOGIN_CONFIGURATION_ID = Config.loginConfID;

    private static final String radiusMethod = "RADIUS";
    private static final String libertyMethod = "Liberty";

    private static final String authnInfoElemDelim = "/";
    private static final String authnInfoValueDelim = "=";
    private static final String authnInfoNameElem = "id";
    private static final String authnInfoPassElem = "password";
    private static final String authnInfoIdElem = "domain";

    private void error(Writer out, String message) throws java.io.IOException {
	out.write("<html>");
	out.write("<head><title>Liberty authentication error</title></head>");
	out.write("<body>" + message + "</body>");
	out.write("<html>");
	return;
    }

    private static LibertyUserSession getLibertySession(HttpServletRequest httpReq) {
	HttpSession ses = httpReq.getSession(true);
	LibertyUserSession lus;
	lus = (LibertyUserSession) ses.getAttribute("liberty.session");
	if (lus == null) {
	    lus = new LibertyUserSession();
	    ses.setAttribute("liberty.session", lus);
	}
	return lus;
    }    

    private static AuthnControlSession getControlSession(HttpServletRequest req) {
        HttpSession ses = req.getSession(true);
        AuthnControlSession ctls;
        ctls = (AuthnControlSession) ses.getAttribute("control.session");
        if (ctls == null) {
            ctls = new AuthnControlSession();
            ses.setAttribute("control.session", ctls);
        }
        return ctls;
    }

    protected void doGet(HttpServletRequest httpReq, HttpServletResponse httpResp)
              throws ServletException
    {
        DebugUtils.debugTime("LoginServlet", "Entering doGet",
                             DebugUtils.HIGH_LEVEL_FUNC);

	String id = httpReq.getParameter("_liberty_id");
	String internetAccess = httpReq.getParameter("_internet_access");
	if (Config.debug) {
            System.err.println("LoginServlet doGet id:" + id);
            System.err.println("internetAccess:" + internetAccess);
        }
	boolean loginPage = (httpReq.getParameter("_login_page") != null);

	handle(httpReq, httpResp, null, null, id, internetAccess, null, null,
               null, loginPage);

        DebugUtils.debugTime("LoginServlet", "Leaving doGet",
                             DebugUtils.HIGH_LEVEL_FUNC);
    }

    protected void doPost(HttpServletRequest httpReq, HttpServletResponse httpResp)
              throws ServletException {
        /*
	 * We only get the name and password here so that attempts to
	 * send them over GET fail; this way if the administrator
	 * messes up the HTML form in the login page by making it do a GET
	 * instead of a POST, it is obviously broken, rather than just
	 * insecure.
	 * XXX: Should also check for TLS and refuse name and password
	 * in that case too.
	 */
        DebugUtils.debugTime("LoginServlet", "Entering doPost",
                             DebugUtils.HIGH_LEVEL_FUNC);

	boolean loginPage = (httpReq.getParameter("_login_page") != null);
	String autoLogin = httpReq.getParameter("_auto_login");
        if (autoLogin == null) {
	    String name = httpReq.getParameter("_authn_name");
	    String pass = httpReq.getParameter("_authn_pass");
	    String id = httpReq.getParameter("_liberty_id");
	    String internetAccess = httpReq.getParameter("_internet_access");
	    String mac = httpReq.getParameter("_mac_addr");
	    String key = httpReq.getParameter("_l2_key");
	    if (Config.debug) {
		System.err.println("LoginServlet doPost id:" + id);
		System.err.println("name:" + name + " pass:" + pass);
                System.err.println("internetAccess:" + internetAccess);
                System.err.println("mac address:" + mac);
                System.err.println("L2 session key:" + key);
            }
	    handle(httpReq, httpResp, name, pass, id, internetAccess, mac, key,
                   null, loginPage);
	} else {
	    String anpMessage = httpReq.getParameter("_anp_message");
	    if (Config.debug) {
                System.err.println("LoginServlet doPost anp message:" + 
                                   anpMessage);
            }
	    handle(httpReq, httpResp, null, null, null, null, null, null,
                   anpMessage, loginPage);
        }

        DebugUtils.debugTime("LoginServlet", "Leaving doPost",
                             DebugUtils.HIGH_LEVEL_FUNC);
    }


    private void handle(HttpServletRequest httpReq,
                        HttpServletResponse httpResp,
                        String name, String pass, String id,
                        String internetAccess,
                        final String mac, final String key,
                        String anpMessage,
                        boolean loginPage)
            throws ServletException
    {
        DebugUtils.debugTime("LoginServlet", "Entering handle",
                             DebugUtils.LOW_LEVEL_FUNC);

	try {
            /* Process ANPMessage if received */
            if (handleANP(httpReq, httpResp, anpMessage)) {
                    return;
            }
            DebugUtils.debugTime("LoginServlet", "ANP Message checked",
                                 DebugUtils.LOW_LEVEL_FUNC);

            /* Process data entered by the user if available*/
            if (handleManualLogin(httpReq, httpResp, name, pass, id,
                                  internetAccess, mac, key)) {
                return;
            }
            DebugUtils.debugTime("LoginServlet", "Manual login checked",
                                 DebugUtils.LOW_LEVEL_FUNC);

            /* If redirection from another page because user is not
             * authenticated, try to seamlessly authenticate if an IDP is
             * provided by the user using a cookie
             */
            if (processCookies(httpReq, httpResp)) {
                return;
            }
            DebugUtils.debugTime("LoginServlet", "Cookies checked",
                                 DebugUtils.LOW_LEVEL_FUNC);

	    /*
	     * Forward request to the login page.  The login page should
	     * cause the user agent to POST back the necessary auth data
	     * back to this servlet.
	     */
             if (loginPage || (Config.autoLoginPage == null)) {
                 forwardPage(httpReq, httpResp, Config.loginPage);
             } else {
	         /* Build Authentication Capabilities Response */
                 String authnCap = 
		     ANPAuthnCapabilitiesManager.getAuthnCapabilitiesRespStr();
                 httpReq.getSession(true).setAttribute("anp.authn_capabilities",
					           authnCap);
                 forwardPage(httpReq, httpResp, Config.autoLoginPage);
             }

            DebugUtils.debugTime("LoginServlet", "Leaving handle",
                                 DebugUtils.LOW_LEVEL_FUNC);
	}
	catch (Exception e) {
	    throw new ServletException("LoginServlet failed", e);
	}
    }


    private boolean handleANP(HttpServletRequest httpReq,
                              HttpServletResponse httpResp,
                              String anpMessage)
            throws Exception
    {
	if ((anpMessage != null) && (!anpMessage.equals(""))) {
            byte[] anpMessageBits = URLUtils.base64Decode(anpMessage);
            String anpMessageStr = new String(anpMessageBits);
            Document anpMessageDoc = XMLUtils.parse(anpMessageStr);

            ANPRequest anpRequest = new ANPRequest();
            anpRequest.parse(anpMessageDoc.getDocumentElement());
            DebugUtils.debugTime("LoginServlet", "ANPResponse parsed",
                                 DebugUtils.LOW_LEVEL_FUNC);

            if (anpRequest.getQuery() != null) {
                ANPRequestProcessorAction action = null;
                ANPRequestProcResult procResult = new ANPRequestProcResult();
                ANPRequestProcessor arp = new SimpleANPRequestProcessor();
                action = arp.process(httpReq, httpResp, anpRequest, procResult);

                if (action == ANPRequestProcessorAction.RETURN_RESPONSE) {
                    if (procResult.failureCode != null) {
                        forwardPage(httpReq, httpResp, Config.loginPage);
                    } else {
                        if (procResult.queryType.equals(
                            ANPRequestProcResult.AUTHN_CAPB_QUERY)) {
                            //handleAuthnCapabilitiesQuery(httpReq, httpResp);
	                    /* Build Authentication Capabilities Response */
                            String authnCap = ANPAuthnCapabilitiesManager.getAuthnCapabilitiesRespStr();
                            httpReq.getSession(true).setAttribute(
                                "anp.authn_capabilities", authnCap);
                            forwardPage(httpReq, httpResp,
                                        Config.autoLoginPage);
                        } else if (procResult.queryType.equals(
                            ANPRequestProcResult.AUTHN_QUERY)) {
                            handleAuthnQuery(httpReq, httpResp, procResult);
                        } else {
                            throw new Exception();
                        }
                    }
                } else if (action == ANPRequestProcessorAction.FAILED) {
                    throw new Exception();
                } else {
                    throw new Exception();
                }
            } else {
                ANPResponse anpResponse = new ANPResponse();
                anpResponse.parse(anpMessageDoc.getDocumentElement());
                DebugUtils.debugTime("LoginServlet", "ANPResponse parsed",
                                     DebugUtils.LOW_LEVEL_FUNC);
                //ANPResponseProcessor arp = new BrowserANPResponseProcessor();
                //arp.process(httpReq, httpResp, anpResponse);
                // This code is missing
            }

            DebugUtils.debugTime("LoginServlet", "Leaving handleANP",
                                 DebugUtils.LOW_LEVEL_FUNC);
            return true;
        } else {
            return false;
        }
    }


    private boolean handleManualLogin(HttpServletRequest httpReq,
                                      HttpServletResponse httpResp,
                                      String name, String pass, String id,
                                      String internetAccess, 
                                      final String mac, final String key)
            throws Exception
    {
        boolean iAccess = false;
        if ((internetAccess != null) && (internetAccess.equals("yes"))) {
            iAccess = true;
        }

        if (id != null) {
            if (id.equals("_liberty_local")) {
                if ((name != null) && (pass != null) && (!name.equals(""))) {
                    if (Config.debug) {
                        System.err.println("Handling Liberty Local Authentication");
                    }
                    handleLocal(httpReq, httpResp, name, pass, iAccess, mac, key);
                    return true;
                }
            } else {
                if (id != null) {
                    if (Config.debug) {
                        System.err.println("Handling Liberty IDP Authentication");
                    }
                    boolean result = handleLiberty(httpReq, httpResp, id, iAccess, mac, key);
                    return result;
                }
            }
        } else {
            if ((name != null) && (pass != null) && (!name.equals(""))) {
                HttpSession ses = httpReq.getSession(true);
                if (ses.getAttribute("radius.authenticated") == null) {
                    if (Config.debug) {
                        System.err.println("Handling RADIUS Authentication");
                    }
                    handleRadius(httpReq, httpResp, name, pass, iAccess, mac, key);
                }
                else {
                    forwardPage(httpReq, httpResp, Config.alreadyAuthnPage);
                }
                return true;
            }
        }
        return false;
    }
                      

    public void handleAuthnCapabilitiesQuery (HttpServletRequest httpReq, HttpServletResponse httpResp) {
    }


    public void handleAuthnQuery (HttpServletRequest httpReq, HttpServletResponse httpResp, ANPRequestProcResult procResult) {
    }


    public boolean processCookies (HttpServletRequest httpReq,
                                   HttpServletResponse httpResp)
           throws ServletException
    {
        Cookie[] receivedCookies = httpReq.getCookies();
        if (Config.debug) {
            System.err.println("Cookies retrieved");
        }

        if (receivedCookies != null) {
            String pendingIDP = null;
            int len = receivedCookies.length;
            for (int i=0; i < len; i++) {
                if (receivedCookies[i].getName().equals("idp")) {
                    pendingIDP = (String) receivedCookies[i].getValue();
                    if (Config.debug) {
                        System.err.println("\"idp\" cookie value: " +
                                           pendingIDP);
                    }
                    /* As the user comes from another network, we assume he is
                     * roaming and desires access to the external network
                     */
                    if (handleLiberty(httpReq, httpResp, pendingIDP, true,
                                      null, null)) {
                        return true;
                    }
                }
            }
            if (Config.debug) {
                System.err.println("No valid \"idp\" cookie received");
            }
        }
        else {
            if (Config.debug) {
                System.err.println("No cookies received");
            }
        }
        return false;
    }


    private void handleRadius(HttpServletRequest httpReq, 
                              HttpServletResponse httpResp,
                              final String name, String pass, boolean iAccess,
                              final String mac, final String key)
            throws ServletException
    {
        DebugUtils.debugTime("LoginServlet", "Entering handleRadius",
                             DebugUtils.LOW_LEVEL_FUNC);

	try {
            /* Authenticate the user */
            AuthenticatorResult authn = null;
            authn = SessionRadiusAuthenticator.authenticate(httpReq,
                                                            name, pass);
            DebugUtils.debugTime("LoginServlet", "RADIUS authentication finished",
                                 DebugUtils.LOW_LEVEL_FUNC);

            /* Evaluate the authentication result */
            boolean res = evaluateAuthnResult(authn, httpReq, httpResp,
                                              iAccess, mac, key, null,
                                              Config.successfulRadiusAuthnPage,
                                              Config.failedAuthnPage);
            if (!res) {
                /* Remove the session information that indicates successful authn */
                SessionRadiusAuthenticator.sessionCleanUp(httpReq);
            }
	}
	catch (Exception e) {
	    throw new ServletException("LoginServlet -RADIUS Authentication- failed",
                                       e);
	}
    }


    private void handleLocal(HttpServletRequest httpReq, HttpServletResponse httpResp,
                             final String name, String pass, boolean iAccess,
                             final String mac, final String key)
            throws ServletException
    {
        DebugUtils.debugTime("LoginServlet", "Entering handleLocal",
                             DebugUtils.LOW_LEVEL_FUNC);

	try {
            /* Authenticate the user */
            AuthenticatorResult authn = null;
            authn = SessionLocalAuthenticator.authenticate(httpReq,
                                                           name, pass);
            DebugUtils.debugTime("LoginServlet", "LOCAL authentication finished",
                                 DebugUtils.LOW_LEVEL_FUNC);

            /* Evaluate the authentication result */
            boolean res = evaluateAuthnResult(authn, httpReq, httpResp,
                                              iAccess, mac, key, null,
                                              Config.successfulLibertyAuthnPage,
                                              Config.failedAuthnPage);
            if (!res) {
                /* Remove the session information that indicates successful authn */
                SessionLocalAuthenticator.sessionCleanUp(httpReq);
            }
	}
	catch (Exception e) {
	    throw new ServletException("LoginServlet -LOCAL Authentication- failed",
                                       e);
	}
    }


    private boolean handleLiberty(HttpServletRequest httpReq,
                                  HttpServletResponse httpResp, 
                                  String id, boolean iAccess,
                                  final String mac, final String key) 
            throws ServletException
    {
        DebugUtils.debugTime("LoginServlet", "Entering handleLiberty",
                             DebugUtils.LOW_LEVEL_FUNC);

	try {
            /* Authenticate the user */
            AuthenticatorResult authn = null;
            String redirectURL[] = new String[2];
              // Two values are returned: the first contains the URL for
              // redirection; the second, the URL for redirection without
              // query part
            authn = LibertyAuthenticator.authenticate(httpReq,
                                                      id, redirectURL);
            DebugUtils.debugTime("LoginServlet", "LIBERTY authentication finished",
                                 DebugUtils.LOW_LEVEL_FUNC);

            /* Evaluate the authentication result */
            /* Liberty Authentication fails if the IDP is unknown */
            /* Indicate that the authentication could not be done */
            boolean res = evaluateAuthnResult(authn, httpReq, httpResp,
                                              iAccess, mac, key, redirectURL,
                                              Config.successfulLibertyAuthnPage,
                                              null);
            if (!res) {
                if (Config.debug) {
                    System.err.println("LIBERTY authentication failed");
                }
                /* Remove the session information that indicates successful authn */
                LibertyAuthenticator.sessionCleanUp(httpReq);
            }

            return res;
	}
	catch (Exception e) {
	    throw new ServletException("LoginServlet -LIBERTY Authentication- failed",
                                       e);
	}
    }


    private boolean evaluateAuthnResult(AuthenticatorResult authn,
                                     HttpServletRequest httpReq, 
                                     HttpServletResponse httpResp,
                                     boolean iAccess,
                                     String mac,
                                     String key,
                                     String redirectURL[],
                                     String successPage,
                                     String failurePage)
            throws ServletException
    {
	try {
            AuthenticatorResult L2authn = null;
            boolean res = true;

            /* Evaluate the authentication result */
	    if (authn == AuthenticatorResult.AUTHN_SUCCESS) {
                if (Config.debug) {
                    System.err.println(
                        "LoginServlet: successful authentication");
                }

                /* Validate L2 info */
                if (Config.compoundAuthn) {
                    L2authn = SimpleL2Authenticator.authenticate(mac, key);
                    DebugUtils.debugTime("LoginServlet", "L2 info validation finished",
                                         DebugUtils.LOW_LEVEL_FUNC);
                } else {
                    L2authn = AuthenticatorResult.AUTHN_SUCCESS;
                    if (Config.debug) {
                        System.err.println(
                            "LoginServlet: L2 info validation skipped");
                    }
                }

                /* Evaluate the L2 validation result */
                if (L2authn == AuthenticatorResult.AUTHN_SUCCESS) {
                    if (Config.debug) {
                        System.err.println(
                            "LoginServlet: successful L2 info validation");
                    }

                    /* If requested, allow user to access the external network */
                    if (iAccess) {
                        if (Config.debug) {
                            System.err.println("LoginServlet: remote IP must be added to the filter");
                        }
                        FirewallController.giveAccess(httpReq);
                    }

                   /* Show result page or, if none defined, requested page */ 
                    if (successPage != null) {
                        forwardPage(httpReq, httpResp, successPage);
                    } else {
		        String uri = httpReq.getRequestURI();
		        String cont = httpReq.getContextPath();
		        String path = uri.substring(cont.length());
                        forwardPage(httpReq, httpResp, path);
                    }
                } else if (L2authn == AuthenticatorResult.AUTHN_FAILED) {
                    if (Config.debug) {
                        System.err.println(
                            "LoginServlet: L2 info validation failed");
                    }

                    /* Show result page or, if none defined, remain in same page */ 
                    if (failurePage != null) {
                        forwardPage(httpReq, httpResp, failurePage);
		    }
                    res = false;
                } else {
                    if (Config.debug) {
                        System.err.println(
                            "LoginServlet: Incorrect L2 info validation result");
                    }

                    throw new Exception();
                }
            } else if (authn == AuthenticatorResult.AUTHN_FAILED) {
                if (Config.debug) {
                    System.err.println(
                        "LoginServlet: Authentication failed");
                }

                /* Show result page or, if none defined, remain in same page */ 
                if (failurePage != null) {
                    forwardPage(httpReq, httpResp, failurePage);
		}
                res = false;
            } else if (authn == AuthenticatorResult.AUTHN_REDIRECT) {
                if (Config.debug) {
                    System.err.println("LoginServlet: Must continue authentication through another server");
                }

                if (redirectURL == null) {
	            throw new Exception("LoginServlet - Missing redirect URL information");
                }

                /* Firewall must allow the client access the IDP */
                if (Config.debug) {
                    System.err.println("LoginServlet: remote IP and URL \"" +
                        redirectURL[1] + "\" must be added to the filter");
                }
                FirewallController.giveURLAccess(httpReq, redirectURL[1]);

                /* Remember whether client must be given access to the
                   external netowork upon successful authentication */
                AuthnControlSession ctls = getControlSession(httpReq);
                ctls.setExternalAccess(iAccess);

                /* Redirect client to redirect URL */
                if (Config.debug) {
                    System.err.println("Redirecting to: " + redirectURL[0]);
                }
                httpResp.sendRedirect(redirectURL[0]);
            } else {
                if (Config.debug) {
                    System.err.println(
                        "LoginServlet: Incorrect authentication result");
                }

                throw new Exception();
            }

            DebugUtils.debugTime("LoginServlet", "Leaving evaluateAuthnResult",
                                 DebugUtils.LOW_LEVEL_FUNC);
           return res;
	}
	catch (Exception e) {
	    throw new ServletException("LoginServlet -Evaluating Authentication result failed",
                                       e);
	}
    }


    private boolean forwardPage(HttpServletRequest httpReq,
                                HttpServletResponse httpResp, 
                                String path) 
            throws Exception
    {
        /* Forward to the appropiate page */
        if (path != null) {
            ServletConfig cfg = getServletConfig();
            ServletContext ctx = cfg.getServletContext();
            RequestDispatcher disp;
            disp = ctx.getRequestDispatcher(path);
            disp.forward(httpReq, httpResp);
            if (Config.debug) {
                System.err.println("ForwardPage: Forwarding to: " + path);
            }
            return true;
        }
        else {
            if (Config.debug) {
                System.err.println("ForwardPage: No page received as parameter");
            }
            return false;
        }
    }
}
