/*
 * $Id: SSOServlet.java,v 1.6.2.1 2002/09/17 01:52:34 jsergent Exp $
 * SSORequest.java
 *
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.experimentalstuff.com/Technologies/IPL/download.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2002 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): Ana Sanz Merino [University of California Berkeley, 2003]
 */

package com.sun.liberty.servlets;
import com.sun.liberty.AuthnRequestProcessor;
import com.sun.liberty.Config;
import com.sun.liberty.LibertyArtifactAuthnResponse;
import com.sun.liberty.LibertyArtifactMessage;
import com.sun.liberty.LibertyAuthnRequest;
import com.sun.liberty.LibertyAuthnResponse;
import com.sun.liberty.LibertyElement;
import com.sun.liberty.LibertyHTMLPostAuthnResponse;
import com.sun.liberty.LibertyHTMLPostMessage;
import com.sun.liberty.LibertyProfile;
import com.sun.liberty.LibertyURLAuthnRequest;
import com.sun.liberty.LibertyUser;
import com.sun.liberty.LibertyUserSession;
import com.sun.liberty.ProviderDescriptorFactory;
import com.sun.liberty.RequestProcessorAction;
import com.sun.liberty.SPDescriptor;
import com.sun.liberty.SimpleAuthnRequestProcessor;
import java.io.Writer;
import java.security.KeyStore;
import javax.security.auth.Subject;
import javax.security.auth.callback.Callback;
import javax.security.auth.callback.CallbackHandler;
import javax.security.auth.callback.NameCallback;
import javax.security.auth.callback.PasswordCallback;
import javax.security.auth.callback.UnsupportedCallbackException;
import javax.security.auth.login.LoginContext;
import javax.security.auth.login.LoginException;
import javax.servlet.RequestDispatcher;
import javax.servlet.ServletConfig;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import ucb.eecs.sahara.sso.DebugUtils;


/**
 * Servlet to handle incoming URLQueryAuthnRequests at the IDP.
 * The client to this servlet is a user agent that was redirected here by a
 * service provider.
 * 
 * @author Jonathan Sergent
 * @version $Revision: 1.6.2.1 $
 */

public class SSOServlet extends HttpServlet {
    public static final String LOGIN_CONFIGURATION_ID = Config.loginConfID;
    
    private String authRedirectURL;

    private void error(Writer out, String message) throws java.io.IOException {
        out.write("<html>");
        out.write("<head><title>Liberty authentication error</title></head>");
        out.write("<body>" + message + "</body>");
        out.write("<html>");
        return;
    }

    private static LibertyUserSession getLibertySession(HttpServletRequest req) {
        HttpSession ses = req.getSession(true);
        LibertyUserSession lus;
        lus = (LibertyUserSession) ses.getAttribute("liberty.session");
        if (lus == null) {
            lus = new LibertyUserSession();
            ses.setAttribute("liberty.session", lus);
        }
	return lus;
    }    

    private void handle(HttpServletRequest req, HttpServletResponse resp,
                        boolean attempted, boolean consented) throws ServletException {
        DebugUtils.debugTime("SSOServlet", "Entering handle",
                             DebugUtils.LOW_LEVEL_FUNC);
	try {
	    Writer out = resp.getWriter();

	    LibertyURLAuthnRequest arq = new LibertyURLAuthnRequest();
	    try {
		arq.parse(req.getQueryString());
	    } catch (java.security.SignatureException e) {
		e.printStackTrace();
		error(out, "Signature validation error.");
		return;
	    }

            DebugUtils.debugTime("SSOServlet", "Request parsed",
                                 DebugUtils.LOW_LEVEL_FUNC);

            ProviderDescriptorFactory dir;
            dir = ProviderDescriptorFactory.getInstance();
	    SPDescriptor sp;
	    sp = (SPDescriptor) dir.find(arq.getProviderID());
	    if (sp == null) {
		error(out, "Cannot find provider: " + arq.getProviderID());
		return;
	    }

            DebugUtils.debugTime("SSOServlet", "SP checked",
                                 DebugUtils.LOW_LEVEL_FUNC);

	    HttpSession ses = req.getSession(true);
	    LibertyUserSession lus = getLibertySession(req);
	    LibertyUser lu = lus.getUser(); // might be null if not authnd yet

	    SimpleAuthnRequestProcessor sarp;
	    sarp = new SimpleAuthnRequestProcessor();

	    LibertyAuthnResponse ars = null;
	    LibertyProfile profile = arq.getProtocolProfile();

	    if (profile == LibertyProfile.BRWS_POST) {
		ars = new LibertyHTMLPostAuthnResponse();
	    } else if (profile == LibertyProfile.BRWS_ART) {
		ars = new LibertyArtifactAuthnResponse();
	    } else {
		throw new Exception("Protocol profile not implemented");
	    }

	    RequestProcessorAction action;
	    action = sarp.process(arq, ars, attempted, consented, lus);

            DebugUtils.debugTime("SSOServlet", "Request processed",
                                 DebugUtils.LOW_LEVEL_FUNC);

	    ServletConfig cfg = getServletConfig();
	    ServletContext ctx = cfg.getServletContext();

	    // XXX: This is too long
	    if (action == RequestProcessorAction.RETURN_RESPONSE) {
		java.net.URL u = sp.getAssertionConsumerURL();
		if (u == null) {
		    error(out, "Cannot find destination URL.");
		    return;
		}
		String dest = u.toString();
		if (profile == LibertyProfile.BRWS_POST) {
		    LibertyHTMLPostMessage html;
		    html = (LibertyHTMLPostMessage) ars;
		    String fragment = html.getHTMLFragment(dest);
		    req.getSession(true).setAttribute(
			    "liberty.html_fragment", fragment);
		    if (Config.debug) {
			System.err.println("SSOServlet fragment:" + fragment);
		    }
		    ctx.getRequestDispatcher("/post.jsp").forward(req, resp);

                    DebugUtils.debugTime("SSOServlet", "Dispatching",
                                         DebugUtils.LOW_LEVEL_FUNC);

		    return;
		} else if (profile == LibertyProfile.BRWS_ART) {
		    LibertyArtifactMessage art;
		    art = (LibertyArtifactMessage) ars;
		    String query = art.getArtifactQueryFor(sp.getProviderID());
		    String url = dest + '?' + query;
		    if (Config.debug) {
			System.err.println("SSOServlet: sendRedirect:" + url);
		    }
		    resp.sendRedirect(url);

                    DebugUtils.debugTime("SSOServlet", "Redirecting",
                                         DebugUtils.LOW_LEVEL_FUNC);

		    return;
		} else {
		    throw new Exception("Profile not implemented: "
			    + profile);
		}
		/* NOTREACHED */
	    } else if (action == RequestProcessorAction.ATTEMPT_USER_AUTH) {
		/*
		 * Forward request to the login page.  The login page
		 * should cause the user agent to POST back the necessary
		 * auth data back to this servlet.  Our doPost will do
		 * JAAS magic to attempt to authenticate the user with
		 * this information.
		 */
		if (Config.debug) {
		    System.err.println("SSOServlet: forward to login");
		}
		ctx.getRequestDispatcher(Config.loginPage).forward(req, resp);

                DebugUtils.debugTime("SSOServlet", "Dispatching",
                                     DebugUtils.LOW_LEVEL_FUNC);

		return;
	    } else if (action == RequestProcessorAction.OBTAIN_USER_CONSENT) {
		if (Config.debug) {
		    System.err.println("SSOServlet: forward to consent");
		}
		ctx.getRequestDispatcher("/consent.jsp").forward(req, resp);
		return;
	    } else if (action == RequestProcessorAction.FAILED) {
		throw new Exception("Authentication failed");
	    } else {
		throw new Exception("Unknown request processing action");
	    }
	} catch (Exception e) {
            throw new ServletException("Error processing AuthnRequest", e);
        }
    }

    public void doGet(HttpServletRequest req, HttpServletResponse resp) throws ServletException {
        DebugUtils.debugTime("SSOServlet", "Entering doGet",
                             DebugUtils.HIGH_LEVEL_FUNC);

	handle(req, resp, false, false);

        DebugUtils.debugTime("SSOServlet", "Leaving doGet",
                             DebugUtils.HIGH_LEVEL_FUNC);
    }

    public void doPost(HttpServletRequest req, HttpServletResponse resp) throws ServletException {
        DebugUtils.debugTime("SSOServlet", "Entering doPost",
                             DebugUtils.HIGH_LEVEL_FUNC);

	String name = req.getParameter("_authn_name");
	String pass = req.getParameter("_authn_pass");
	String consented = req.getParameter("_liberty_consented");
	boolean attemptedAuthn = false;
	boolean agreed = false;
	
        if (name != null && pass != null) {
	    attemptedAuthn = true;
	    attemptAuthn(req, name, pass);
	}

	agreed = (consented != null && consented.equals("agree"));
	handle(req, resp, attemptedAuthn, agreed);

        DebugUtils.debugTime("SSOServlet", "Leaving doPost",
                             DebugUtils.HIGH_LEVEL_FUNC);
    }


    private static void attemptAuthn(HttpServletRequest req, final String name, final String pass) throws ServletException {
        DebugUtils.debugTime("SSOServlet", "Entering attemptAuthn",
                             DebugUtils.LOW_LEVEL_FUNC);

	Subject subj = null;

try {
	if (!Config.debugAuthn) {
	    class MyCallbackHandler implements CallbackHandler {
		public void handle(Callback[] callbacks)
			throws UnsupportedCallbackException {
		    for (int i = 0; i < callbacks.length; i++) {
			// XXX: Should we support more callbacks here?  Maybe.
			Callback c = callbacks[i];
			if (c instanceof NameCallback) {
			    NameCallback nc = (NameCallback) c;
			    nc.setName(name);
			} else if (c instanceof PasswordCallback) {
			    PasswordCallback pc = (PasswordCallback) c;
			    pc.setPassword(pass.toCharArray());
			} else {
			    throw new UnsupportedCallbackException
				(c, "unrecognized callback");
			}
		    }
		}
	    }
	    CallbackHandler myCallback = new MyCallbackHandler();
	    try {
		LoginContext lctx = new LoginContext(LOGIN_CONFIGURATION_ID,
			myCallback);
		lctx.login();
		subj = lctx.getSubject();
	    } catch (LoginException e) {
		e.printStackTrace();
	    }
	} else {
	    if (Config.debug) {
		System.out.println("SSOServlet: debugAuthn performed");
            }
	    /* Hack! Log in any user where username == pass. */
	    if (name.equals(pass)) {
		subj = new Subject();
		java.security.Principal pr;
		pr = new com.sun.security.auth.SolarisPrincipal(name);
		subj.getPrincipals().add(pr);
	    }
	}

	LibertyUserSession lus = getLibertySession(req);

	HttpSession ses = req.getSession(true);
	if (subj == null) {
	    /*
	     * if auth failed, clears any previous identity since
	     * it is no longer valid.
	     * XXX: should we initiate a SingleLogout in this case?
	     */
	    ses.removeAttribute("liberty.session");
	} else {
	    try {
		lus.setUser(subj);
		if (lus.getUser() == null) {
		    throw new Exception("setUser(subj) failed");
		}
	    } catch (Exception e) {
		ses.removeAttribute("liberty.session");
		throw new ServletException("Cannot set user.", e);
	    }
	}
} catch (Exception e) {
System.err.println(e);
}
    }
}
