package ucb.eecs.sahara.sso.anp;

import ucb.eecs.sahara.sso.anp.ANPElement;
import com.sun.liberty.XMLElement;
import com.sun.liberty.XMLUtils;
import java.util.Iterator;
import java.util.Date;
import java.util.List;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;


/**
 *
 * @author Ana Sanz Merino [University of California Berkeley, 2003]
 * @version $Revision: 1.1.1.1 $
 */


public abstract class ANPAbstractMessage extends XMLElement implements ANPElement {
    private int majorVersion = 1;
    private int minorVersion = 0;
    private String signedBy = null;
    private String receivedFrom = null;

    public void parse(Element e) {
	setMajorVersion(Integer.valueOf(e.getAttributeNS(null, "MajorVersion")).intValue());
	setMinorVersion(Integer.valueOf(e.getAttributeNS(null, "MinorVersion")).intValue());

	Element signature = null;
	NodeList list = e.getChildNodes();
	for (int i = 0; i < list.getLength(); i++) {
	    if (list.item(i) instanceof org.w3c.dom.Text) { continue; }
	    Element child = (Element) list.item(i);
	    if (isElement(child, DSIG_NS_URI, "Signature")) {
		signature = child;
	    }
	}

	if (signature != null) {
	    setSignedBy(XMLUtils.verify(signature, receivedFrom) ? receivedFrom : null);
	}
    }

    protected void addAttributesTo(Element s) {
	s.setAttributeNS(null, "MajorVersion", new Integer(majorVersion).toString());
	s.setAttributeNS(null, "MinorVersion", new Integer(minorVersion).toString());
    }

    protected void addChildrenTo(Element s) {
	if (signedBy != null) {
	    XMLUtils.sign(s);
	}
    }

    public int getMajorVersion() {
	return majorVersion;
    }

    public void setMajorVersion(int value) {
	majorVersion = value;
    }

    public int getMinorVersion() {
	return minorVersion;
    }

    public void setMinorVersion(int value) {
	minorVersion = value;
    }

    public String getSignedBy() {
	return signedBy;
    }

    public void setSignedBy(String value) {
	signedBy = value;
    }

    public void setReceivedFrom(String value) {
	receivedFrom = value;
    }
}
