package ucb.eecs.sahara.sso.anp;

import ucb.eecs.sahara.sso.DebugUtils;
import ucb.eecs.sahara.sso.anp.ANPAuthnCapabilitiesStatement;
import com.sun.liberty.saml.SAMLPStatus;
import com.sun.liberty.XMLUtils;
import com.sun.liberty.URLUtils;
import com.sun.liberty.Config;

import java.util.List;
import java.util.LinkedList;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/**
 * @author Ana Sanz Merino
 * @version $Revision: 1.1.1.1 $
 */

public class ANPAuthnCapabilitiesManager {

    private static String authnCapabilitiesRespStr = null;
    private static ANPResponse authnCapabilitiesResp = null;
    private static ANPAuthnCapabilitiesStatement authnCapabilitiesStatement = null;

    static {
        try {
            generateAuthnCapabilitiesRespStr();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public static synchronized void generateAuthnCapabilitiesRespStr()
			       throws Exception {
        DebugUtils.debugTime("ANPAuthnCapabilitiesManager", "Entering generateAuthnCapabilitiesRespStr",
                             DebugUtils.LOW_LEVEL_FUNC);

        generateAuthnCapabilitiesResp();

        String respStr = XMLUtils.serialize(authnCapabilitiesResp);
        authnCapabilitiesRespStr = URLUtils.base64Encode(respStr.getBytes());
        DebugUtils.debugTime("ANPAuthnCapabilitiesManager",
                             "Authentication Capabilities Response string generated",
                             DebugUtils.LOW_LEVEL_FUNC);
    }

    public static synchronized void generateAuthnCapabilitiesResp()
			       throws Exception {
        DebugUtils.debugTime("ANPAuthnCapabilitiesManager", "Entering generateAuthnCapabilitiesResp",
                             DebugUtils.LOW_LEVEL_FUNC);

        generateAuthnCapabilitiesStatement();
        authnCapabilitiesResp = new ANPResponse();
        authnCapabilitiesResp.setMajorVersion(1);
        authnCapabilitiesResp.setMinorVersion(0);
        SAMLPStatus status = new SAMLPStatus();
        status.setStatusCode("samlp:Success");
        authnCapabilitiesResp.setStatus(status);
        List statements = new LinkedList();
        statements.add(authnCapabilitiesStatement);
        authnCapabilitiesResp.setStatements(statements);
        authnCapabilitiesResp.setSignedBy(Config.localProviderID);
    }

    public static synchronized void generateAuthnCapabilitiesStatement()
			       throws Exception {
        DebugUtils.debugTime("ANPAuthnCapabilitiesManager",
                             "Entering generateAuthnCapabilitiesStatement",
                             DebugUtils.LOW_LEVEL_FUNC);

        Document doc = XMLUtils.parseFile(Config.authnCapabilitiesFile);
        if (Config.debug) {
            System.err.println("Authentication Capabilities file parsed");
        }

        authnCapabilitiesStatement = new ANPAuthnCapabilitiesStatement();
        authnCapabilitiesStatement.parse(doc.getDocumentElement());
        DebugUtils.debugTime("ANPAuthnCapabilitiesManager",
                             "Authentication Capabilities Statement generated",
                             DebugUtils.LOW_LEVEL_FUNC);
    }

    public static synchronized String getAuthnCapabilitiesRespStr() {
        return authnCapabilitiesRespStr;
    }

    public static synchronized ANPResponse getAuthnCapabilitiesResp() {
        return authnCapabilitiesResp;
    }

    public static synchronized ANPAuthnCapabilitiesStatement getAuthnCapabilitiesStatement() {
        return authnCapabilitiesStatement;
    }
}

