package ucb.eecs.sahara.sso.anp;

import com.sun.liberty.XMLElement;
import java.util.List;
import java.util.LinkedList;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/**
 * @author Ana Sanz Merino
 * @version $Revision: 1.1.1.1 $
 */

public class ANPIDPGroup extends XMLElement implements ANPElement {
    private String specialIDPList = null;
    private ANPIDPList idpList= null;
    private List authnMethodIDReferences = null;
    private List chargingOptionIDReferences = null;

    public void parse(Element e) {
        NodeList list = e.getChildNodes();
        for (int j = 0; j < list.getLength(); j++) {
            if (list.item(j) instanceof org.w3c.dom.Text) { continue; }
            Element child = (Element) list.item(j);
            if (isElement(child, ANP_NS_URI, "SpecialIDPList")) {
                setSpecialIDPList(child.getFirstChild().getNodeValue());
            } else if (isElement(child, ANP_NS_URI, "IDPList")) {
                ANPIDPList c = new ANPIDPList();
                c.parse(child);
                setIDPList(c);
            } else if (isElement(child, ANP_NS_URI, "AuthnMethodIDReference")) {
                List set = getAuthnMethodIDReferences();
                if (set == null) {
                    set = new LinkedList();
                }
                set.add(child.getFirstChild().getNodeValue());
                setAuthnMethodIDReferences(set);
            } else if (isElement(child, ANP_NS_URI, "ChargingOptionIDReference")) {
                List set = getChargingOptionIDReferences();
                if (set == null) {
                    set = new LinkedList();
                }
                set.add(child.getFirstChild().getNodeValue());
                setChargingOptionIDReferences(set);
            }
        }
    }

    public Element getElement(Document doc) {
	Element l = doc.createElementNS(ANP_NS_URI, "anp:IDPGroup");
	addChildrenTo(l);
	return l;
    }

    protected void addChildrenTo(Element s) {
        if (specialIDPList != null) {
	    appendNewChildNSWithText(s, ANP_NS_URI,
                ANP_NS_ID + ":SpecialIDPList", specialIDPList);
        }
        if (idpList != null) {
            s.appendChild(idpList.getElement(s.getOwnerDocument()));
        }
        iterateAddNewChildrenNSWithText(s, ANP_NS_URI, ANP_NS_ID + ":AuthnMethodIDReference",
                                 authnMethodIDReferences);
        iterateAddNewChildrenNSWithText(s, ANP_NS_URI, ANP_NS_ID + ":ChargingOptionIDReference",
                                 chargingOptionIDReferences);
    }

    public String getSpecialIDPList() {
	return specialIDPList;
    }

    public void setSpecialIDPList(String value) {
	specialIDPList = value;
    }

    public ANPIDPList getIDPList() {
	return idpList;
    }

    public void setIDPList(ANPIDPList value) {
	idpList = value;
    }

    public List getAuthnMethodIDReferences() {
	return authnMethodIDReferences;
    }

    public void setAuthnMethodIDReferences(List value) {
	authnMethodIDReferences = value;
    }

    public List getChargingOptionIDReferences() {
	return chargingOptionIDReferences;
    }

    public void setChargingOptionIDReferences(List value) {
	chargingOptionIDReferences = value;
    }
}
