package ucb.eecs.sahara.sso.anp;

import ucb.eecs.sahara.sso.anp.ANPElement;
import com.sun.liberty.saml.SAMLStatement;
import com.sun.liberty.saml.SAMLElement;
import com.sun.liberty.saml.SAMLPStatus;
import java.util.List;
import java.util.LinkedList;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/**
 * Data bindings for the anp:Response element.  Get and set methods
 * correspond to child elements and attributes in the XML schema for this
 * element.
 *
 * @author Ana Sanz Merino
 * @version $Revision: 1.1.1.1 $
 */

public class ANPResponse extends ANPAbstractMessage {
    SAMLPStatus status;
    List statements;

    public void parse(Element e) {
	NodeList list = e.getChildNodes();
	for (int i = 0; i < list.getLength(); i++) {
	    if (list.item(i) instanceof org.w3c.dom.Text) { continue; }
	    Element child = (Element) list.item(i);
            SAMLStatement statementToAdd = null;
	    if (isElement(child, SAMLElement.SAMLP_NS_URI, "Status")) {
		SAMLPStatus status = new SAMLPStatus();
		status.parse(child);
		setStatus(status);
            } else if (isElement(child, ANP_NS_URI, 
                       "AuthnCapabilitiesStatement")) {
                statementToAdd = new ANPAuthnCapabilitiesStatement();
            } else if (isElement(child, ANP_NS_URI, "AuthnStatement")) {
                statementToAdd = new ANPAuthnStatement();
            }

            if (statementToAdd != null) {
                statementToAdd.parse(child);
                List statements = getStatements();
                if (statements == null) {
                    statements = new LinkedList();
                }
                statements.add(statementToAdd);
                setStatements(statements);
            }
	}
	super.parse(e);
    }

    public Element getElement(Document doc) {
	Element r;
	r = doc.createElementNS(ANP_NS_URI, ANP_NS_ID + ":Response");
	addAttributesTo(r);
	addChildrenTo(r);
	return r;
    }

    protected void addAttributesTo(Element s) {
	super.addAttributesTo(s);
    }

    protected void addChildrenTo(Element s) {
	iterateAddChildren(s, statements);
	if (status != null) {
	    s.appendChild(status.getElement(s.getOwnerDocument()));
	}
	super.addChildrenTo(s);
    }

    public List getStatements() {
	return statements;
    }

    public void setStatements(List value) {
	statements = value;
    }

    public SAMLPStatus getStatus() {
	return status;
    }

    public void setStatus(SAMLPStatus value) {
	status = value;
    }
}
