package ucb.eecs.sahara.sso.anp;

import ucb.eecs.sahara.sso.DebugUtils;

import com.sun.liberty.Config;

import com.sun.liberty.SoapUtils;
import com.sun.liberty.XMLElement;
import javax.xml.soap.SOAPMessage;
import javax.xml.soap.SOAPConnection;
import javax.xml.soap.SOAPConnectionFactory;
import javax.xml.soap.MessageFactory;
import javax.xml.transform.dom.DOMSource;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * Utility functions for SOAP processing using SAAJ.
 *
 * @author Ana Sanz Merino
 * @version $Revision: 1.1.1.1 $
 */
public class ANPSoapUtils extends SoapUtils{

    static SOAPConnectionFactory connFactory;
    static MessageFactory msgFactory;

    static {
        try {
            msgFactory = MessageFactory.newInstance();
            connFactory = SOAPConnectionFactory.newInstance();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public static void addToBody(ANPSoapMessage msg, SOAPMessage soap) {
	Element body = getBodyElement(soap);
	Document doc = body.getOwnerDocument();
	body.appendChild(((XMLElement) msg).getElement(doc));
	try {
	    soap.getSOAPPart().setContent(new DOMSource(doc));
	} catch (Exception e) {
	    e.printStackTrace();
	    throw new RuntimeException("setContent() failed--see console");
	}
    }

    /**
     * Sends the ANP message over SOAP with an empty SOAP header
     * to a SOAP endpoint
     * @param msg the Liberty message
     * @param url the SOAP endpoint URL
     */
    public static SOAPMessage send(ANPSoapMessage msg, String url) {
	try {
	    SOAPConnection conn = connFactory.createConnection();
            DebugUtils.debugTime("ANPSoapUtils", "SOAP Connection created",
                                 DebugUtils.LOW_LEVEL_FUNC);
	    SOAPMessage soap = msgFactory.createMessage();
            DebugUtils.debugTime("ANPSoapUtils", "SOAP message created",
                                 DebugUtils.LOW_LEVEL_FUNC);
	    msg.addToBody(soap);
            DebugUtils.debugTime("ANPSoapUtils", "Message content added",
                                 DebugUtils.LOW_LEVEL_FUNC);
	    if (Config.debug) {
		soap.writeTo(System.err);
	    }
	    return conn.call(soap, url);
	    /*
	     * Note that any SOAP faults will be indicated in the reply
	     * message and should be checked by the caller...
	     */
	} catch (Exception e) {
	    e.printStackTrace();
	    return null;
	}
    }
}
