package ucb.eecs.sahara.sso.anp;

import com.sun.liberty.Config;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import java.util.List;
import java.util.Iterator;

import ucb.eecs.sahara.sso.authn.LibertyAuthenticator;
import ucb.eecs.sahara.sso.authn.AuthenticatorResult;


/**
 * Processes ANPAuthnQueries for LIBERTY.
 * @author Ana Sanz Merino [University of California Berkeley, 2003]
 * @version $Revision: 1.1.1.1 $
 */

public class SimpleLibertyANPAuthnQueryProcessor implements ANPAuthnQueryProcessor {

    private static final String idpNameParam = "loginInfo/idpName";

    public ANPRequestProcessorAction process(HttpServletRequest httpReq,
					     HttpServletResponse httpResp,
					     ANPAuthnQuery authnQuery,
                                             ANPAuthnQueryProcResult procResult)
	   throws Exception
    {
        try {
            String idpName = null;

            /* Extract user information from the ANPAuthnQuery */
            Iterator i = null;
            List userInfo = authnQuery.getUserInfo();
            i = userInfo.iterator();
            while (i.hasNext()) {
                ANPUserInfo ui = (ANPUserInfo) i.next();
                if (ui.getAttributeName().equals(idpNameParam)) {
                    idpName = (String) ui.getAttributeValues().iterator().next();
                }
            }
            if (Config.debug) {
                System.err.println("LIBERTY requested with idp " + idpName);
            }

            /* Check all required user info was present in the ANPAuthnQuery */
            if (idpName == null) {
                System.err.println("Missing required user information for LIBERTY authentication");
                procResult.failureCode = "samlp:Sender";
                return ANPRequestProcessorAction.RETURN_RESPONSE;
            }

            /* Authenticate the user */
            String redirectURL[] = new String[2];
            // Two values are returned: the first contains the URL for
            // redirection; the second, the URL for redirection without
            // query part
            AuthenticatorResult authnResult = 
                LibertyAuthenticator.authenticate(httpReq, idpName,
                                                  redirectURL);

            if (authnResult == AuthenticatorResult.AUTHN_SUCCESS) {
                procResult.authnQueryResult = new ANPAuthnQueryResult();
                procResult.authnQueryResult.authnResult = "Successful";
            } else if (authnResult == AuthenticatorResult.AUTHN_FAILED) {
                procResult.authnQueryResult = new ANPAuthnQueryResult();
                procResult.authnQueryResult.authnResult = "Failed";
            } else if (authnResult == AuthenticatorResult.AUTHN_REDIRECT) {
                if (Config.debug) {
                    System.err.println("LIBERTY authentication: Must authenticate through IDP");
                    System.err.println("Redirect URL: " + redirectURL[0]);
                }
                procResult.authnQueryResult = new ANPAuthnQueryResult();
                procResult.authnQueryResult.authnResult = "Redirect";
                procResult.authnQueryResult.redirectURL = redirectURL[0];
                procResult.authnQueryResult.redirectServer = redirectURL[1];
            } else {
                if (Config.debug) {
                    System.err.println("Error at LIBERTY authentication");
                }
                procResult.failureCode = "samlp:Receiver";
            }
            return ANPRequestProcessorAction.RETURN_RESPONSE;
        } catch (Exception e) {
            if (Config.debug) {
                System.err.println(e);
            }
            return ANPRequestProcessorAction.FAILED;
        }
    }
}

