package ucb.eecs.sahara.sso.authn;

import com.sun.liberty.Config;

import java.net.DatagramSocket;
import java.net.DatagramPacket;
import java.net.InetAddress;

/**
 * Validates compound authentication information.
 *
 * @author Ana Sanz Merino
 * @version $Revision: 1.1.1.1 $
 */

public class SimpleL2Authenticator {

    private static final String macParam = "MacAddr";
    private static final String keyParam = "Key";
    private static final int bufferSize = 64;

    public static AuthenticatorResult authenticate(final String macAddress,
                                                   final String sessionKey)
                  throws Exception
    {
        /* Verify L2 session key digest */
        String data = "MAC_KEY_VERIFY " + macAddress + " " + sessionKey;
        if (Config.debug) {
            System.err.println("Data to send: " + data);
        }
        String res = sendRecvDatagram(data);
        if (Config.debug) {
            System.err.println("Validation of session key result: " + res + "\n");
        }
        if (res.equals("OK")) { 
            return AuthenticatorResult.AUTHN_SUCCESS;
        } else {
            if (Config.debug) {
                System.err.println("Failed L2 session key verification");
            }
            return AuthenticatorResult.AUTHN_FAILED;
        }
    }

    private static String sendRecvDatagram (String data)
    {
        try {
            // Open socket
            DatagramSocket sock = new DatagramSocket();

            // Send request
            byte[] buff = data.getBytes();
            DatagramPacket pack = new DatagramPacket(buff, buff.length,
                                      InetAddress.getLocalHost(), Config.L2RemotePort);
            sock.send(pack);

            // Receive answer
            byte[] buff2 = new byte[bufferSize];
            int num_recv = -1;
            DatagramPacket pack2 = new DatagramPacket(buff2, buff2.length);
            if (Config.debug) {
                System.err.println("Waiting for response...");
            }
            sock.receive(pack2);
            num_recv = pack2.getLength();
            if (Config.debug) {
                System.err.println("Num bytes received: "  + num_recv + "\n");
            }

            // Close socket
            sock.close();

            return new String(buff2, 0, num_recv);
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }
}
